<?php 
/**
 * NOTICE OF LICENSE.
 *
 * This source module is subject to the following license: REGULAR LICENSE for one domain
 *
 * DISCLAIMER
 *
 * Not for sale.
 *
 * @author    Envía Ya S.A. de C.V
 * @copyright Envía Ya S.A. de C.V https://enviaya.com.mx
 * @license   REGULAR LICENSE
 */

class EnviayaServiceRate extends ObjectModel
{


    /**
     * Customer delivery address.
     *
     * @var int
     */
    public $id_address_delivery;

    /**
     * Related customer cart.
     *
     * @var int
     */
    public $id_cart;

    /**
     * Selected Service Point details.
     *
     * @var string
     */
    public $details;

    /**
     * Hashed id Service Point.
     *
     * @var string
     */
    public $hash;


    /**
     * Saved response from order request.
     *
     * @var string
     */
    public $order;


    /**
     * Additional order data.
     *
     * @var string
     */
    public $order_data;


    /**
     * Usually empty, table for extra info.
     *
     * @var string
     */
    public $extra_data;

    /**
     * Selected Price (service) by customer.
     *
     * @var string
     */
    public $selected;

    /**
     * Initial request.
     *
     * @var string
     */
    public $requestsaved;

    /**
     * Booking  result.
     *
     * @var string
     */
    public $booking;

    /**
     * Booking data result.
     *
     * @var string
     */
    public $booking_data;

    /**
     * Date added.
     */
    public $date_add;

    /**
     * Last updated date.
     */
    public $date_upd;

    /**
     * Standard `ObjectModel` definition. Contains the database fields this
     * entity has.
     *
     * @var array
     */
    public static $definition = array(
        'table' => 'enviaya_service_rates',
        'primary' => 'id_service_rate',
        'multilang' => false,
        'fields' => array(
            'id_cart' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'id_address_delivery' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'details' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'hash' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'selected' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'requestsaved' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'booking' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'booking_data' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'order' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'order_data' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'extra_data' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
        ),
    );

     /**
     * Retrieve a `EnviayaServicePoint` instance related to the `$cart_id`, or
     * a new instance to relate to the given `$cart_id`.
     *
     * @return EnviayaServicePoint
     */
    public static function getFromCart($cart_id)
    {
        $query = 'SELECT id_service_rate
        FROM `'._DB_PREFIX_.self::$definition['table'].'`
        WHERE id_cart="'.(int) $cart_id.'"';
        $id = Db::getInstance()->getValue($query);

        $service_rate = new self($id ? $id : null);
        $service_rate->id_cart = (int) $cart_id;

        return $service_rate;
    }

    /**
     * Parse the JSON data with the service point information and return an
     * object.
     *
     * @return stdClass|null returns `null` if there're no details
     */
    public function getDetails()
    {
        if (!$this->details) {
            return null;
        }

        return Tools::jsonDecode($this->details, true);
    }

    public function getSelected()
    {
        //error_log("Enviaya: inPoint debug Selected in lib ".print_r($this->selected, true), 0);
        if (!$this->selected) {
            return null;
        }

        return Tools::jsonDecode($this->selected, true);
    }

    public function getRequestsaved()
    {
        if (!$this->requestsaved) {
            return null;
        }

        return Tools::jsonDecode($this->requestsaved, true);
        //return Tools::jsonDecode($this->requestsaved, true, 512, JSON_INVALID_UTF8_IGNORE);
    }

    public function getBoxsaved()
    {
        if (!$this->box) {
            return null;
        }

        return Tools::jsonDecode($this->box, true);
        //return Tools::jsonDecode($this->requestsaved, true, 512, JSON_INVALID_UTF8_IGNORE);
    }


    public function getCustomsDeclsaved()
    {
        if (!$this->customsdeclaration) {
            return null;
        }

        return Tools::jsonDecode($this->customsdeclaration, true);
        //return Tools::jsonDecode($this->requestsaved, true, 512, JSON_INVALID_UTF8_IGNORE);
    }

    public function getCustomsDeclDatasaved()
    {
        if (!$this->customsdeclaration_data) {
            return null;
        }

        return Tools::jsonDecode($this->customsdeclaration_data, true);
        //return Tools::jsonDecode($this->requestsaved, true, 512, JSON_INVALID_UTF8_IGNORE);
    }

    public function getOrderSaved()
    {
        if (!$this->order) {
            return null;
        }

        return Tools::jsonDecode($this->order, true);
        //return Tools::jsonDecode($this->requestsaved, true, 512, JSON_INVALID_UTF8_IGNORE);
    }

    public function getOrderDataSaved()
    {
        if (!$this->order_data) {
            return null;
        }

        return Tools::jsonDecode($this->order_data, true);
        //return Tools::jsonDecode($this->requestsaved, true, 512, JSON_INVALID_UTF8_IGNORE);
    }

    public function getExtraDatasaved()
    {
        if (!$this->extra_data) {
            return null;
        }

        return Tools::jsonDecode($this->extra_data, true);
        //return Tools::jsonDecode($this->requestsaved, true, 512, JSON_INVALID_UTF8_IGNORE);
    }

    public function getBooking()
    {
        if (!$this->booking) {
            return null;
        }

        return Tools::jsonDecode($this->booking, true);
        //this may return null if some of the json data not valid!
    }

    public function geTracking()
    {
        if (!$this->extra_data) {
            return null;
        }

        return Tools::jsonDecode($this->extra_data, true);
        //this may return null if some of the json data not valid!
    }

    public function getBookingData()
    {
        if (!$this->booking_data) {
            return null;
        }

        return Tools::jsonDecode($this->booking_data, true);
        //this may return null if some of the json data not valid!
    }

    public function getBookingDataObj()
    {
        if (!$this->booking_data) {
            return null;
        }

        return Tools::jsonDecode($this->booking_data);
        //for PDF encoded in responce
    }

}