<?php
/**
 * Controller responsible for saving the service point selection in the database
 * after the selection.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade to newer
 * versions in the future. https://devdocs.prestashop.com/1.7/modules/concepts/controllers/front-controllers/
 * <ModuleName> => enviayacarrier
 * <FileName> => servicerateselection.php
 * Format expected: <ModuleName><FileName>ModuleFrontController
 *
 * @author    Envía Ya S.A. de C.V
 * @copyright Envía Ya S.A. de C.V https://enviaya.com.mx
 * @license   REGULAR LICENSE
 */

//psr
class EnviayacarrierServicerateselectionModuleFrontController extends ModuleFrontController
{
    public function init()
    {
        
        parent::init();
        $cart = $this->context->cart;
        require_once dirname(__FILE__, 3).'/api/loader.php';

        $token = Tools::getValue('token');
        
        //prevent nonAJAX calls
        if (!Tools::isSubmit('ajax') || !$cart || Tools::getToken(false)!=$token) {
            $this->httpResponseCode(404);
            $this->ajaxDie(false);
        }
        
        $module = $this->module;
        $selected_price = Tools::getValue('selected_price');
        $selected_data = Tools::getValue('selected_data'); //can be 0! coz array index
        
        if (empty($selected_price) || !isset($selected_data)) {

            $this->httpResponseCode(400);
            $this->ajaxDie(Tools::jsonEncode(array(
                'error' => $this->l('no_service_point'),
            )));
        }


        $service_rate = EnviayaServiceRate::getFromCart($cart->id);
        
        $selected_id_in_array = (int)$selected_data;
        $response = unserialize($service_rate->getDetails());
        
        $service_rate->selected = json_encode(serialize($response['rates'][$selected_id_in_array]));
        $test = unserialize($service_rate->getSelected());
        
        $service_rate->save();
       

        $this->httpResponseCode(201);
        $this->ajaxDie('');
    }

    public static function httpResponseCode($code = null)
    {
        $statuses = array(
            '100' => 'Continue',
            '101' => 'Switching Protocols',
            '200' => 'OK',
            '201' => 'Created',
            '202' => 'Accepted',
            '203' => 'Non-Authoritative Information',
            '204' => 'No Content',
            '205' => 'Reset Content',
            '206' => 'Partial Content',
            '300' => 'Multiple Choices',
            '301' => 'Moved Permanently',
            '302' => 'Moved Temporarily',
            '303' => 'See Other',
            '304' => 'Not Modified',
            '305' => 'Use Proxy',
            '400' => 'Bad Request',
            '401' => 'Unauthorized',
            '402' => 'Payment Required',
            '403' => 'Forbidden',
            '404' => 'Not Found',
            '405' => 'Method Not Allowed',
            '406' => 'Not Acceptable',
            '407' => 'Proxy Authentication Required',
            '408' => 'Request Time-out',
            '409' => 'Conflict',
            '410' => 'Gone',
            '411' => 'Length Required',
            '412' => 'Precondition Failed',
            '413' => 'Request Entity Too Large',
            '414' => 'Request-URI Too Large',
            '415' => 'Unsupported Media Type',
            '500' => 'Internal Server Error',
            '501' => 'Not Implemented',
            '502' => 'Bad Gateway',
            '503' => 'Service Unavailable',
            '504' => 'Gateway Time-out',
            '505' => 'HTTP Version not supported',
        );

        $code = null === $code ? '200' : $code;
        $text = isset($statuses[$code]) ? $statuses[$code] : $statuses['200'];

        if (function_exists('http_response_code')) {
            return http_response_code((int) $code);
        } else {
            $protocol = (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0');
            header($protocol.' '.$code.' '.$text);

            return $code;
        }
    }
}
