<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to the following license: REGULAR LICENSE
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade to newer
 * versions in the future.
 *
 * @author    Envía Ya S.A. de C.V
 * @copyright Envía Ya S.A. de C.V https://enviaya.com.mx
 * @license   REGULAR LICENSE
 */

if (!defined('_PS_VERSION_')) {
    exit;
}


class EnviayaCarrier extends CarrierModule
{

    public $id_carrier;
    private $_html = '';
    private $_moduleName = 'enviayacarrier';
    protected $postErrors = array();
    private $_fieldsList = array();

    public function __construct()
    {
        
        $this->name = 'enviayacarrier';
        $this->tab = 'shipping_logistics';
        $this->version = '2.1.0';
        $this->author = 'Envía Ya S.A. de C.V';
        $this->need_instance = 0;
        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'EnvíaYa! for PrestaShop';
        $this->description = $this->l('plugin_description');
        $this->confirmUninstall = $this->l('uninstall_warning');
        $this->ps_versions_compliancy = array(
            'min' => '1.6',
            'max' => _PS_VERSION_,
        );
        
        $this->registerHook('displayAdminOrderTop');
        $this->registerHook('displayAdminOrderTabContent');
      
    }

    public function uninstall()
    {   

        
        if (!parent::uninstall()
            || !$this->uninstallSQL()
        ){
            return false;
        }

        

        return true;
    }

    public function install($delete_params = true)
    {
        $id_carrier = false;
        if (false == extension_loaded('curl') )  {
            $this->_errors[] = $this->l('curl_existence');

            return false;
        }
                
        // Remove previous added carrier if any
        $this->deleteExistingCarriers();
        $carrierConfig = array(
            'name' => 'EnvíaYa!',
            'url' => 'https://enviaya.com.mx',
            'id_tax_rules_group' => 0,
            'deleted' => 0,
            'shipping_handling' => false,
            'range_behavior' => 0,
            'delay' => array('en' => 'Envía a la mejor tarifa con todas las paqueterías'),
            'id_zone' => 1,
            'is_module' => true,
            'shipping_external' => true,
            'external_module_name' => $this->_moduleName,
            'need_range' => true,
            'active' => true,
        );

  
        if (!parent::install()
            || !$this->registerHook('displayHeader')
            || !$this->registerHook('displayBackOfficeHeader')
            || !$this->registerHook('displayCarrierExtraContent')
            || !$this->registerHook('displayAdminOrderContentShip')
            || !$this->registerHook('displayAdminOrderTabContent')
            || !$this->installSQL()
            || !Configuration::updateGlobalValue('ENVIAYA_LOG', 0)
            || !Configuration::updateGlobalValue('ENVIAYA_SHIPPING_CARRIER_NAME', 0)
            || !Configuration::updateGlobalValue('ENVIAYA_SHIPPING_SERVICE_NAME', 1)
            || !Configuration::updateGlobalValue('ENVIAYA_SHIPPING_DELIVERY_TIME', 3)
            || !Configuration::updateGlobalValue('ENVIAYA_DISPLAY_CARRIER_LOGO', 1)
            || !Configuration::updateGlobalValue('ENVIAYA_AS_DEFINED_PRICE', 1)
            || !Configuration::updateGlobalValue('ENVIAYA_ENABLE_CONTINGENCY_SHIPPING', 1)
            || !Configuration::updateGlobalValue('ENVIAYA_ENABLE_STANDARD_FLAT_RATE', 1)
            || !Configuration::updateGlobalValue('ENVIAYA_STANDARD_FLAT_RATE_TITLE', $this->l('contingency_standard_shipping_title'))
            || !Configuration::updateGlobalValue('ENVIAYA_STANDARD_FLAT_RATE', 100)
            || !Configuration::updateGlobalValue('ENVIAYA_ENABLE_EXPRESS_FLAT_RATE', 1)
            || !Configuration::updateGlobalValue('ENVIAYA_EXPRESS_FLAT_RATE_TITLE', $this->l('contingency_express_shipping_title'))
            || !Configuration::updateGlobalValue('ENVIAYA_EXPRESS_FLAT_RATE', 150)
            || !Configuration::updateGlobalValue('ENVIAYA_API_TIMEOUT', 30)
            || !Configuration::updateGlobalValue('ENVIAYA_COMPLETE_UNINSTALL', 0)

            
            
        ){
            return false;
        }
        $id_carrier = $this->installExternalCarrier($carrierConfig);
        if (false === $id_carrier) {
            $errortext = $this->l('[EnviaYa] Carrier can not be created.');
            if (class_exists('Logger')) {
                
                PrestaShopLogger::addLog($errortext, 3);
            }

            return false;
        }

        return true;
    }

    /**
     *
     * @return bool true if every entity gets created correctly
     */

    private function installSQL()
    {
        $queries = include dirname(__FILE__).'/sql/install.php';

        foreach ($queries as $query) {
            
            if (!Db::getInstance()->execute($query)) {
                return false;
            }
        }

        return true;
    }

    /**
     *
     * @return bool true if every entity gets deleted correctly
     */

    private function uninstallSQL()
    {


        if( (int)Configuration::get('ENVIAYA_COMPLETE_UNINSTALL')){
            $queries = include dirname(__FILE__).'/sql/uninstall.php';

            foreach ($queries as $query) {
            
                if (!Db::getInstance()->execute($query)) {
                    return false;
                }
            }

        }
        
        return true;

        
    }

    public function getOrderShippingCost($cart, $shippingCost)
    {

        if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
            PrestaShopLogger::addLog('[EnviaYa] Rating API request', 1);
        }

        return $this->getPackageShippingCost($cart, $shippingCost, null);
       
    }

    public function getOrderShippingCostExternal($params)
    {

        return getOrderShippingCost($params, null);
    }

    //Get QUOTES FROM CARRIER API
    public function getPackageShippingCost($cart, $shippingCost, $products)
    {
        
        if (!Module::isEnabled('enviayacarrier')) {
            return false;
        }
        $controller = isset($this->context->controller) ? $this->context->controller : null;
       
        $not_allowed_controllers = array(
            'AdminController',
            'AdminOrdersController',
            'OrderConfirmationController',
            'CartController',
        );
       
        $is_not_allowed = !is_null($controller) && in_array(get_class($controller), $not_allowed_controllers);
                
        if ($is_not_allowed || !$cart) {
            // Get quote just in the order-related controllers. Overwise call will be slow
            return false;
        }
        
        require_once dirname(__FILE__).'/api/loader.php';

        $is_test_mode = (int) Configuration::get('ENVIAYA_TEST_MODE');
        if ($is_test_mode) {
            $api_key =  Configuration::get('ENVIAYA_TEST_API_KEY');
        } else {
            $api_key =  Configuration::get('ENVIAYA_LIVE_API_KEY');
        }
        if (!$api_key) {
            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                PrestaShopLogger::addLog('[EnviaYa] No API Key', 3);
            }

            return false;
        }
           
        // your credentials from https://app.enviaya.com.mx/api_keys
        Enviaya::setApiKey($api_key);
        $carrier = new Carrier($this->id_carrier);
        $idCarrierReference = $carrier->id_reference;
        $customerAddress = new Address($cart->id_address_delivery);
        $customer = new Customer($cart->id_customer); //$customer = $this->context->customer;
        
        if (!Validate::isLoadedObject($customer)) {
            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                PrestaShopLogger::addLog('[EnviaYa] No valid customer or guest', 3);
            }
            return false;
        } else {
            $customer_email = $customer->email;
        }

        if (!Validate::isLoadedObject($customerAddress)) {
            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                PrestaShopLogger::addLog('[EnviaYa] Customer address Invalid: '.Tools::jsonEncode($customerAddress),3);
            }

            return false;
        }

        $excluded_postcodes = Configuration::get('ENVIAYA_EXCLUDED_ZONE_POSTCODES');
        $excluded_regions = Configuration::get('ENVIAYA_EXCLUDED_ZONE_REGIONS');

        if( (isset($excluded_postcodes) && !empty($excluded_postcodes)) ||
            (isset($excluded_regions) && !empty($excluded_regions))
         ) {

            $country = new Country((int) $customerAddress->id_country);
            $zone_id = 'Z'.$country->id_zone;
            $country_id = 'C'.$country->id;
            $state_id = 'S'.$customerAddress->id_state;
            
            $excluded_regions_arr = explode(',',$excluded_regions);
            $excluded_postcodes_arr = explode(',',$excluded_postcodes);
            
            $excluded = false;
            if(in_array($zone_id,$excluded_regions_arr) || in_array($country_id,$excluded_regions_arr) || in_array($state_id,$excluded_regions_arr)){
                $excluded = true;
            }

            if( in_array($customerAddress->postcode,$excluded_postcodes_arr) ){
                $excluded = true;
            }

            
            
            if($excluded){

                if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                    PrestaShopLogger::addLog('[EnviaYa] Customer address excluded by excluded zones configuration.', 2);   
                }

                return false;
            }
           
        }
        
        
        $customerAddressIso = $iso = Country::getIsoById((int) $customerAddress->id_country);

        $origincountryiso = $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE'));
        $origincountry_name = Country::getByIso((string) Configuration::get('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE'), false);
        $orderCurrency = new Currency((int) $cart->id_currency);

        $quotedata = array(
            'enviaya_account' => $this->cleanString(Configuration::get('ENVIAYA_ACCOUNT')),
            'api_key' => $api_key
        );

        $orderTotal = $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS, $products);

        $quotedata['shipment']['shipment_type'] = 'Package';
        $quotedata['shipment']['order_total_amount'] = $orderTotal;
        $quotedata['shipment']['currency'] = $orderCurrency->iso_code;
        
        $parcels = $this->buildParcels($products,$orderCurrency);
        if(!empty($parcels)){
            $quotedata['shipment']['parcels'] = $parcels;
        }

        $quotedata['origin_direction']['country_code'] = $origincountryiso;
        $quotedata['origin_direction']['postal_code'] = $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE'));
        if( Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_1') != false &&  !empty(Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_1')) ){
            $quotedata['origin_direction']['direction_1'] = $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_1'));
        }
        if( Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_2') != false &&  !empty(Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_2')) ){
            $quotedata['origin_direction']['direction_2'] = $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_2'));
        }


        $quotedata['destination_direction']['country_code'] =  $customerAddressIso;
        $quotedata['destination_direction']['postal_code'] = $this->cleanString($customerAddress->postcode);
        if( !empty($customerAddress->address1) ){
            $quotedata['destination_direction']['direction_1'] = $this->cleanString($customerAddress->address1);
        }
        if( !empty($customerAddress->address2) ){
            $quotedata['destination_direction']['direction_2'] = $this->cleanString($customerAddress->address2);
        }
        
        $customerCurrency = new Currency((int) $cart->id_currency);

        
        $quotedata['locale'] = Context::getContext()->language->locale;
        $quotedata['currency'] = $orderCurrency->iso_code;
        $quotedata['order_total_amount'] = $orderTotal;
        
        $quotedata_hash = md5(Tools::jsonEncode($quotedata));
        
        $service_rate = EnviayaServiceRate::getFromCart($cart->id);
        $service_rate_id = $service_rate->id;
        
        if (!$service_rate->id || !$service_rate->details || !$service_rate->hash || $service_rate->hash != $quotedata_hash ){

            $origin_direction = array(
                'postal_code'  => $quotedata['origin_direction']['postal_code'],
                'country_code' => $quotedata['origin_direction']['country_code'],
            );
            if( isset($quotedata['origin_direction']['direction_1']) && !empty($quotedata['origin_direction']['direction_1']) ){
                $origin_direction['origin_direction']['direction_1'] = $quotedata['origin_direction']['direction_1'];
            }
            if( isset($quotedata['origin_direction']['direction_2']) && !empty($quotedata['origin_direction']['direction_2']) ){
                $origin_direction['direction_2'] = $quotedata['origin_direction']['direction_2'];
            }
    
            $destination_direction = array(
                'postal_code'  => $quotedata['destination_direction']['postal_code'],
                'country_code' => $quotedata['destination_direction']['country_code'],
            );
            if( isset($quotedata['destination_direction']['direction_1']) && !empty($quotedata['destination_direction']['direction_1']) ){
                $destination_direction['direction_1'] = $quotedata['destination_direction']['direction_1'];
            }
            if( isset($quotedata['destination_direction']['direction_2']) && !empty($quotedata['destination_direction']['direction_2']) ){
                $destination_direction['direction_2'] = $quotedata['destination_direction']['direction_2'];
            }
            
            $quotedata['timeout'] = (int)Configuration::get('ENVIAYA_API_TIMEOUT') - 1;

            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                PrestaShopLogger::addLog('[EnviaYa] Quote data JUST before API request: '.Tools::jsonEncode($quotedata), 1);
            }
            
            try {
                
                $response = Enviaya_Rate::rating($quotedata);
                if (is_object($response)) {
                    //if rates returned should be an Object so Tools::jsonEncode will not print anything
                    PrestaShopLogger::addLog('[EnviaYa] API Rating response: '.print_r($response, true), 1);
                } else {
                    //if error or no rates
                    PrestaShopLogger::addLog('[EnviaYa] API Rating response: '.Tools::jsonEncode($response), 1);
                }
                

                if (!is_object($response)) {
                    if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                        PrestaShopLogger::addLog('[EnviaYa] No valid JSON response', 3);
                    }

                    return false;
                }

            }catch (exception $e) {
                
                $error_msg = $e->getmessage();

                if (class_exists('Logger')) {
                    PrestaShopLogger::addLog('[EnviaYa] API Rating response error: '.Tools::jsonEncode($error_msg), 3);
                }

                //CONTINGENCY SHIPPING
                if(Configuration::get('ENVIAYA_ENABLE_CONTINGENCY_SHIPPING') == "1"){

                    $dt = new DateTime();
                    $dt->setTimezone(new DateTimeZone('America/Mexico_city'));
                    
                    $errors = json_decode($error_msg,true);

                    //$response = array();

                    //STANDARD FLATE RATE
                    if(Configuration::get('ENVIAYA_ENABLE_STANDARD_FLAT_RATE') == "1"){
                        
                        $standard_flat_rate = array(
                            "rate_id" => strtolower('ENVIAYA_STANDARD_FLAT_RATE'),
                            "shipment_id" => str_pad(mt_rand(1,99999999),8,'0',STR_PAD_LEFT),
                            "dynamic_service_name" => Configuration::get('ENVIAYA_STANDARD_FLAT_RATE_TITLE'),
                            "date" => $dt->format('c'),
                            "carrier" =>  Configuration::get('ENVIAYA_STANDARD_FLAT_RATE_TITLE'),
                            "carrier_service_name" => Configuration::get('ENVIAYA_STANDARD_FLAT_RATE_TITLE'),
                            "carrier_service_code" => 'ENVIAYA_STANDARD_FLAT_RATE_TITLE',
                            "carrier_logo_url" => null,
                            "carrier_logo_url_svg" => null,
                            "estimated_delivery" => null,
                            "est_transit_time_hours" => null,
                            "net_shipping_amount" => null,
                            "net_surcharges_amount" => null,
                            "net_total_amount" => null,
                            "vat_amount" => null,
                            "vat_rate" => null,
                            "total_amount" => floatval( Configuration::get('ENVIAYA_STANDARD_FLAT_RATE') ),
                            "currency" => $orderCurrency->iso_code,
                            "list_total_amount" => null,
                            "list_total_amount_currency" => null,
                            "subsidy_total_amount" => null,
                            "service_terms" => null,
                            "surcharges_break_down" => [],
                            "enviaya_service_name" => Configuration::get('ENVIAYA_STANDARD_FLAT_RATE_TITLE'),
                            "enviaya_service_code" => 'ENVIAYA_STANDARD_FLAT_RATE_TITLE',
                            "additional_configuration" => null    
                        );

                        $response['rates'][] = $standard_flat_rate;

                    }

                    //EXPRESS FLAT RATE
                    if(Configuration::get('ENVIAYA_ENABLE_EXPRESS_FLAT_RATE') == "1"){
                        
                        $express_flat_rate = array(
                            "rate_id" => strtolower('ENVIAYA_EXPRESS_FLAT_RATE'),
                            "shipment_id" => str_pad(mt_rand(1,99999999),8,'0',STR_PAD_LEFT),
                            "dynamic_service_name" => Configuration::get('ENVIAYA_EXPRESS_FLAT_RATE_TITLE'),
                            "date" => $dt->format('c'),
                            "carrier" =>  Configuration::get('ENVIAYA_STANDARD_FLAT_RATE_TITLE'),
                            "carrier_service_name" => Configuration::get('ENVIAYA_EXPRESS_FLAT_RATE_TITLE'),
                            "carrier_service_code" => 'ENVIAYA_EXPRESS_FLAT_RATE_TITLE',
                            "carrier_logo_url" => null,
                            "carrier_logo_url_svg" => null,
                            "estimated_delivery" => null,
                            "est_transit_time_hours" => null,
                            "net_shipping_amount" => null,
                            "net_surcharges_amount" => null,
                            "net_total_amount" => null,
                            "vat_amount" => null,
                            "vat_rate" => null,
                            "total_amount" => floatval( Configuration::get('ENVIAYA_EXPRESS_FLAT_RATE') ),
                            "currency" => $orderCurrency->iso_code,
                            "list_total_amount" => null,
                            "list_total_amount_currency" => null,
                            "subsidy_total_amount" => null,
                            "service_terms" => null,
                            "surcharges_break_down" => [],
                            "enviaya_service_name" => Configuration::get('ENVIAYA_EXPRESS_FLAT_RATE_TITLE'),
                            "enviaya_service_code" => 'ENVIAYA_EXPRESS_FLAT_RATE_TITLE',
                            "additional_configuration" => null    
                        );

                        $response['rates'][] = $express_flat_rate;

                    }

                    if(!empty($response)){

                        $response = Enviaya_Util::convertToEnviayaObject(array_merge($response,$errors), $api_key);

                    }else{
                        return false;
                    }

                }else{

                    return false;

                }

               
            }

        }else{

            $response = unserialize($service_rate->getDetails());
            
            $response_selected = unserialize($service_rate->getSelected());
            
            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                if (is_object($response_selected)) {
                    PrestaShopLogger::addLog('[EnviaYa] Selected rate: '.print_r($response_selected, true), 1);
                } else {
                    PrestaShopLogger::addLog('[EnviaYa] Selected rate: '.Tools::jsonEncode($response_selected), 1);
                }
            }
           
        }
        

        if ($response) {
           
            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                if (is_object($response)) {
                    PrestaShopLogger::addLog('[EnviaYa] API rating response accepted as object.', 1);
                } else {
                    PrestaShopLogger::addLog('[EnviaYa] API rating response accepted: '.Tools::jsonEncode($response), 1);
                }
            }
                        
            $serializedObject = serialize($response);
                
                if (!$this->saveServiceRate($cart, Tools::jsonEncode($serializedObject),$quotedata_hash,Tools::jsonEncode($quotedata))) {
                    if (class_exists('Logger')) {
                        PrestaShopLogger::addLog('[EnviaYa] API rate response can not be saved to cache', 3);
                    }
                }

                if (!empty($response['rates'])) {
                    
                    
                    //if response was saved before and have rates
                    if (!empty($response_selected)) {
                        foreach ($response['rates'] as $qkey => $qvalue) {
                            // search
                            //NOTE: carrier id can be same in few rates so check id and price!
                            if ($qvalue->rate_id == $response_selected->rate_id and $qvalue['total_amount'] == $response_selected['total_amount']) {
                                //start compare saved and latest responces
                                $latestamount = $response['rates'][$qkey]['total_amount'];

                                //insurance and other extra cost may change if customer change currency before placing order so need to compare extas in saved selected vs quote details (last rates with all carriers returned after last getQuote request)
                                //$latestextra1 = $response['rates'][$qkey]['extras']['extra'][0]['amount'];
                            }
                        }
                        
                        if (!isset($latestamount)
                        or
                        $latestamount != $response_selected['total_amount']
                        /*or
                        (isset($latestextra1) and array_key_exists('extras', $response_selected) and array_key_exists('extra', $response_selected['extras']) and array_key_exists(0, $response_selected['extras']['extra']) and $latestextra1 != $response_selected['extras']['extra'][0]['amount'])*/
                        ) {
                            $skipsaved = true;

                        } else {
                            $skipsaved = false;
                        }
                    }

                    if (!empty($response_selected) and !empty($response_selected['total_amount']) and $service_rate->hash == $quotedata_hash and !$skipsaved) {

                        //return delivery cost previously selected by customer
                        //TODO:: compare price by enviayacarrierid in saved and full responces
                        //$response_selected['servicelevel']->token vs $response.....
                        $priceWithTaxes = (float) $response_selected['total_amount'];
                        
                        if( isset($response_selected['additional_configuration']['free_shipping'])  ){
                            $priceWithTaxes = 0.00;
                        }

                        //$priceWithoutTaxes = (float) $response_selected['basePrice'];
                        $enviayaCurrencyCode = $response_selected['currency']; //usually AUD
                    } else {
                        
                        $response_arr = $response->__toArray(true);
                       
                    
                        if (!empty($response) and !empty($response_arr['rates']) and array_key_exists(0, $response_arr['rates'])) {

                            $quote = Enviaya_Util::convertToEnviayaObject($response_arr['rates'][0],$api_key);
                            $priceWithTaxes = (float) $quote['total_amount'];

                            if( isset($quote['additional_configuration']['free_shipping'])  ){
                                $priceWithTaxes = 0.00;
                            }

                            $enviayaCurrencyCode = $quote['currency'];

                            $service_rate = EnviayaServiceRate::getFromCart($cart->id);
                            if ($service_rate->id and $service_rate->hash == $quotedata_hash) {
                                $serializedSelected = serialize($quote);
                                $service_rate->selected = Tools::jsonEncode($serializedSelected);
                                $service_rate->save();
                            }

                        }

                        $idCurrencyEnviaya = Currency::getIdByIsoCode($enviayaCurrencyCode);
                        
                        // If the currency returned by Enviaya does not exist in the shop, we cannot display prices
                        if (!$idCurrencyEnviaya) {
                            if (class_exists('Logger')) {
                                PrestaShopLogger::addLog('[EnviaYa]  Unknown Currency '.Tools::jsonEncode($enviayaCurrencyCode),3);
                            }

                            return false;
                        }
                        $EnviayaCurrency = new Currency((int) $idCurrencyEnviaya);
                        
                        $customerCurrency = new Currency((int) $cart->id_currency);
                        $totalCartWithTaxes = $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS, $products);
                        
                        $totalCartWithTaxes = $totalCartWithTaxes / $customerCurrency->conversion_rate;
                        $idZoneDelivery = Country::getIdZone((int) $customerAddress->id_country);
                        if (!Validate::isLoadedObject($EnviayaCurrency) || !Validate::isLoadedObject($customerCurrency)) {
                            return false;
                        }
                        $handlingFees = Configuration::get('PS_SHIPPING_HANDLING');
                        

                        // Adding handling charges
                        if ($carrier->shipping_handling) {
                            $priceWithTaxes += (float) $handlingFees;
                        }
                        //?skip
                        // Additional Shipping Cost per product
                        foreach ($products as $p) {
                            if (!$p['is_virtual']) {
                                $priceWithTaxes += $p['additional_shipping_cost'] * (int) $p['cart_quantity'];
                            }
                        }
                        
                        // We need to convert the price in the default currency of the shop
                        $priceWithTaxes = $priceWithTaxes / $EnviayaCurrency->conversion_rate;
                        // Then we convert it in the currency of the cart
                        $priceWithTaxes = $priceWithTaxes * $customerCurrency->conversion_rate;
                        //$orderMessage = new CustomerMessage();
                       
                        if(isset($response->subsidy) && $orderTotal >= $response->subsidy->minimum_order_amount){
                            return max($priceWithTaxes - $response->subsidy->total_amount,0);
                        }


                        return $priceWithTaxes;
                        

                    }
                }
           
        }


        //Enviaya return valid Object if no errors and rates are there
        if ($response) {
            //if ($response) {
                
            //$response = (array) Tools::jsonDecode($response, true);
            if (!is_object($response)) {
                $errors = $response;
            } elseif (empty($response['rates'])) {
                //Enviaya response object received but RATES is empty
                $errors = $this->l('no rates in Enviaya response');
            } else {
                //no errors Shiipo Object received and contains array with rates objects
                //$errors = $this->l('Enviaya response object but unknown format');
                //$errors = $this->iSobjctKeyExists('messages', $response);
            }

            if (empty($errors)) {
                //start get QUOTE DATA
                //save received response object to Cache (db)
                $serializedObject = serialize($response);
                //save also the $one_box to get info in admin order view so owner can pack into correct box!
                if (empty($customs_declaration)) {
                    $customs_declaration = '';
                }
                if (!$this->saveServiceRate($cart, Tools::jsonEncode($serializedObject),$quotedata_hash,Tools::jsonEncode($quotedata))) {
                    if (class_exists('Logger')) {
                        PrestaShopLogger::addLog('[EnviaYa] API rate response can not be saved to cache', 3);
                    }
                } 

                //if (array_key_exists('rates', $response)) {
                if (!empty($response['rates'])) {

                    //if response was saved before and have rates
                    if (!empty($response_selected)) {

                        foreach ($response['rates'] as $qkey => $qvalue) {
                            // search
                            //NOTE: carrier id can be same in few rates so check id and price!
                            if ($qvalue->rate_id == $response_selected->rate_id and $qvalue['total_amount'] == $response_selected['total_amount']) {
                                //start compare saved and latest responces
                                $latestamount = $response['rates'][$qkey]['total_amount'];

                                //insurance and other extra cost may change if customer change currency before placing order so need to compare extas in saved selected vs quote details (last rates with all carriers returned after last getQuote request)
                                //$latestextra1 = $response['rates'][$qkey]['extras']['extra'][0]['amount'];
                            }
                        }

                        if (!isset($latestamount)
                        or
                        $latestamount != $response_selected['total_amount']
                        /*or
                        (isset($latestextra1) and array_key_exists('extras', $response_selected) and array_key_exists('extra', $response_selected['extras']) and array_key_exists(0, $response_selected['extras']['extra']) and $latestextra1 != $response_selected['extras']['extra'][0]['amount'])*/
                        ) {
                            $skipsaved = true;
                        } else {
                            $skipsaved = false;
                        }
                    }

                    if (!empty($response_selected) and !empty($response_selected['total_amount']) and $service_rate->hash == $quotedata_hash and !$skipsaved) {
                        //if (!empty($response_selected) and $service_rate->hash == $quotedata_hash and !$skipsaved) {

                        //return delivery cost previously selected by customer
                        //TODO:: compare price by enviayacarrierid in saved and full responces
                        //$response_selected['servicelevel']->token vs $response.....
                        $priceWithTaxes = (float) $response_selected['total_amount'];
                        if( isset($response_selected['additional_configuration']['free_shipping'])  ){
                            $priceWithTaxes = 0.00;
                        }
                        //$priceWithoutTaxes = (float) $response_selected['basePrice'];
                        $enviayaCurrencyCode = $response_selected['currency']; //usually AUD
                    } else {
                        //return first from All in the quote
                        //if (!empty($response) and array_key_exists('rates', $response) and array_key_exists(0, $response['rates']) and array_key_exists('amount', $response['rates'][0])) {
                        //!empty($response['rates']
                        if (!empty($response) and !empty($response['rates']) and array_key_exists(0, $response['rates']) and $this->iSobjctKeyExists('total_amount', $response['rates'][0])) {

                            // code to show all carriers even with zero
                            $quote = $response['rates'][0];
                            $priceWithTaxes = (float) $quote['total_amount'];
                            if( isset($quote['additional_configuration']['free_shipping'])  ){
                                $priceWithTaxes = 0.00;
                            }
                            //$priceWithoutTaxes = (float) $quote['basePrice'];
                            $enviayaCurrencyCode = $quote['currency']; //usually AUD
                            $service_rate = EnviayaCarrierRate::getFromCart($cart->id);
                            if ($service_rate->id and $service_rate->hash == $quotedata_hash) {
                                $serializedSelected = serialize($quote);
                                //$service_rate->selected = Tools::jsonEncode($quote);
                                $service_rate->selected = Tools::jsonEncode($serializedSelected);
                                $service_rate->save();
                            }
                            //END code to show all carriers
                        } else {

                            return false;
                        }
                    }

                    $idCurrencyEnviaya = Currency::getIdByIsoCode($enviayaCurrencyCode);
                    // If the currency returned by Enviaya does not exist in the shop, we cannot display prices
                    if (!$idCurrencyEnviaya) {
                        if (class_exists('Logger')) {
                            PrestaShopLogger::addLog('[EnviaYa]  Unknown Currency '.Tools::jsonEncode($enviayaCurrencyCode),3);
                        }

                        return false;
                    }
                    $EnviayaCurrency = new Currency((int) $idCurrencyEnviaya);
                    $customerCurrency = new Currency((int) $cart->id_currency);
                    $totalCartWithTaxes = $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS, $products);
                    $totalCartWithTaxes = $totalCartWithTaxes / $customerCurrency->conversion_rate;
                    $idZoneDelivery = Country::getIdZone((int) $customerAddress->id_country);
                    if (!Validate::isLoadedObject($EnviayaCurrency) || !Validate::isLoadedObject($customerCurrency)) {
                        return false;
                    }
                    $handlingFees = Configuration::get('PS_SHIPPING_HANDLING');
                    // Adding handling charges
                    if ($carrier->shipping_handling) {
                        $priceWithTaxes += (float) $handlingFees;
                    }
                    //?skip
                    // Additional Shipping Cost per product
                    foreach ($products as $p) {
                        if (!$p['is_virtual']) {
                            $priceWithTaxes += $p['additional_shipping_cost'] * (int) $p['cart_quantity'];
                        }
                    }
                    // We need to convert the price in the default currency of the shop
                    $priceWithTaxes = $priceWithTaxes / $EnviayaCurrency->conversion_rate;
                    // Then we convert it in the currency of the cart
                    $priceWithTaxes = $priceWithTaxes * $customerCurrency->conversion_rate;
                    //$orderMessage = new CustomerMessage();

                    if(isset($response['subsidy']) && $totalCartWithTaxes >= $response['subsidy']['minimum_order_amount']){
                        return max($priceWithTaxes - $response['subsidy']['total_amount'],0);
                    }

                    return $priceWithTaxes;
                //END SUCCESS RESPONSE
                } else {
                    if (class_exists('Logger')) {
                        PrestaShopLogger::addLog('[EnviaYa] API rate response without rates', 3);
                    }

                    return false;
                }
            } else {

                return false;
            }
        } else {
            //no responce or response is not an object ||blocked by firewall or curl cant connect
            return false;
        }
        //direct return price //return 15;
    }

    public function deleteExistingCarriers()
    {
        Db::getInstance()->update('carrier', array('deleted' => 1), 'external_module_name = "'.$this->name.'"');
    }

    public static function installExternalCarrier($config)
    {
        $id_lang = (int) Context::getContext()->language->id;
        $carrier = new Carrier();
        $carrier->name = $config['name'];
        $carrier->url = $config['url'];
        $carrier->id_tax_rules_group = $config['id_tax_rules_group'];
        $carrier->id_zone = $config['id_zone'];
        $carrier->active = $config['active'];
        $carrier->deleted = $config['deleted'];
        $carrier->delay = $config['delay'];
        $carrier->shipping_handling = $config['shipping_handling'];
        $carrier->range_behavior = $config['range_behavior'];
        $carrier->is_module = $config['is_module'];
        $carrier->shipping_external = $config['shipping_external'];
        $carrier->external_module_name = $config['external_module_name'];
        $carrier->need_range = $config['need_range'];
        $carrier->active = $config['active'];

        $languages = Language::getLanguages(true);
        foreach ($languages as $language) {
            if (('FR' == $language['iso_code']) || ('en' == $language['iso_code'])) {
                $carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
            }
            if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) && isset($config['delay'][$language['iso_code']])) {
                $carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
            } elseif ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) && isset($config['delay']['en'])) {
                $carrier->delay[(int) $language['id_lang']] = $config['delay']['en'];
            }
        }

        if ($carrier->add()) {
            $groups = Group::getGroups(true);
            foreach ($groups as $group) {
                if (Tools::version_compare(_PS_VERSION_, '1.7', '>=')) {
                    Db::getInstance()->insert(
                        'carrier_group',
                        array('id_carrier' => (int) $carrier->id, 'id_group' => (int) $group['id_group'])
                    );
                } else {
                    Db::getInstance()->autoExecute(_DB_PREFIX_.'carrier_group', array('id_carrier' => (int) ($carrier->id), 'id_group' => (int) ($group['id_group'])), 'INSERT');
                }
            }

            $rangePrice = new RangePrice();
            $rangePrice->id_carrier = $carrier->id;
            $rangePrice->delimiter1 = '0';
            $rangePrice->delimiter2 = '10000';
            $rangePrice->add();

            $rangeWeight = new RangeWeight();
            $rangeWeight->id_carrier = $carrier->id;
            $rangeWeight->delimiter1 = '0';
            $rangeWeight->delimiter2 = '10000';
            $rangeWeight->add();
            //domestic and international delivery
            $zones = Zone::getZones(true);
            foreach ($zones as $zone) {
                $carrier->addZone((int) $zone['id_zone']);
            }

            if (Tools::version_compare(_PS_VERSION_, '1.7', '>=')) {
                Db::getInstance()->insert(
                    'delivery',
                    array('id_carrier' => (int) ($carrier->id), 'id_range_price' => (int) ($rangePrice->id), 'id_range_weight' => null, 'id_zone' => (int) ($carrier->id_zone), 'price' => '0'),
                    true
                );
                Db::getInstance()->insert(
                    'delivery',
                    array('id_carrier' => (int) ($carrier->id), 'id_range_price' => null, 'id_range_weight' => (int) ($rangeWeight->id), 'id_zone' => (int) ($carrier->id_zone), 'price' => '0'),
                    true
                );
            } else {
                Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_.'delivery', array('id_carrier' => (int) ($carrier->id), 'id_range_price' => (int) ($rangePrice->id), 'id_range_weight' => null, 'id_zone' => (int) ($carrier->id_zone), 'price' => '0'), 'INSERT');

                Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_.'delivery', array('id_carrier' => (int) ($carrier->id), 'id_range_price' => null, 'id_range_weight' => (int) ($rangeWeight->id), 'id_zone' => (int) ($carrier->id_zone), 'price' => '0'), 'INSERT');
            }

            if (Tools::version_compare(_PS_VERSION_, '1.7', '>=')) {
                $logoPath = _PS_MODULE_DIR_.'enviayacarrier/views/img/enviaya_carrier_17.png';
            } else {
                $logoPath = _PS_MODULE_DIR_.'enviayacarrier/views/img/enviaya_carrier.png';
            }
            //JPG is critical here
            $destination1 = _PS_SHIP_IMG_DIR_.(int) $carrier->id.'.jpg';
            $destination2 = _PS_TMP_IMG_DIR_.'carrier_mini_'.(int) $carrier->id.'_'.$id_lang.'.jpg';
            Tools::copy($logoPath, $destination1);
            Tools::copy($logoPath, $destination2);
            // Return ID Carrier
            return (int) ($carrier->id);
        }

        return false;
    }

    public function getContent()
    {

        require_once dirname(__FILE__).'/api/loader.php';

        if (Tools::isSubmit('btnSubmit') ) {
           
            $configuration_values = array(
                'ENVIAYA_SENDER_ADDRESS_ID' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_ID'),
                'ENVIAYA_SENDER_ADDRESS_COMPANY' => rtrim(Tools::getValue('ENVIAYA_SENDER_ADDRESS_COMPANY')),
                'ENVIAYA_SENDER_ADDRESS_FULL_NAME' => rtrim(Tools::getValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME')),
                'ENVIAYA_SENDER_ADDRESS_POSTAL_CODE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE'),
                'ENVIAYA_SENDER_ADDRESS_CITY' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_CITY'),
                'ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD'),
                'ENVIAYA_SENDER_ADDRESS_DIRECTION_1' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1'),
                'ENVIAYA_SENDER_ADDRESS_DIRECTION_2' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2'),
                'ENVIAYA_SENDER_ADDRESS_PHONE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_PHONE'),
                'ENVIAYA_SENDER_ADDRESS_EMAIL' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_EMAIL'),
                'ENVIAYA_SENDER_ADDRESS_STATE_CODE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE'),
                'ENVIAYA_SENDER_ADDRESS_STATE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE'),
                'ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE'),
                'ENVIAYA_SENDER_ADDRESS_COUNTRY' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY'),
                'ENVIAYA_LIVE_API_KEY' => Tools::getValue('ENVIAYA_LIVE_API_KEY'),
                'ENVIAYA_TEST_API_KEY' => Tools::getValue('ENVIAYA_TEST_API_KEY'),
                'ENVIAYA_TEST_MODE' => Tools::getValue('ENVIAYA_TEST_MODE'),
                'ENVIAYA_ACCOUNT' => Tools::getValue('ENVIAYA_ACCOUNT'),
                'ENVIAYA_ACCOUNT_ID' => Tools::getValue('ENVIAYA_ACCOUNT_ID'),
                'ENVIAYA_SHIPPING_CARRIER_NAME' => Tools::getValue('ENVIAYA_SHIPPING_CARRIER_NAME'),
                'ENVIAYA_SHIPPING_SERVICE_NAME' => Tools::getValue('ENVIAYA_SHIPPING_SERVICE_NAME'),
                'ENVIAYA_SHIPPING_DELIVERY_TIME' => Tools::getValue('ENVIAYA_SHIPPING_DELIVERY_TIME'),
                'ENVIAYA_DISPLAY_CARRIER_LOGO' => Tools::getValue('ENVIAYA_DISPLAY_CARRIER_LOGO'),
                'ENVIAYA_AS_DEFINED_PRICE' => Tools::getValue('ENVIAYA_AS_DEFINED_PRICE'),
                'ENVIAYA_ENABLE_CONTINGENCY_SHIPPING' => Tools::getValue('ENVIAYA_ENABLE_CONTINGENCY_SHIPPING'),
                'ENVIAYA_ENABLE_STANDARD_FLAT_RATE' => Tools::getValue('ENVIAYA_ENABLE_STANDARD_FLAT_RATE'),
                'ENVIAYA_STANDARD_FLAT_RATE_TITLE' => Tools::getValue('ENVIAYA_STANDARD_FLAT_RATE_TITLE'),
                'ENVIAYA_STANDARD_FLAT_RATE' => Tools::getValue('ENVIAYA_STANDARD_FLAT_RATE'),
                'ENVIAYA_ENABLE_EXPRESS_FLAT_RATE' => Tools::getValue('ENVIAYA_ENABLE_EXPRESS_FLAT_RATE'),
                'ENVIAYA_EXPRESS_FLAT_RATE_TITLE' => Tools::getValue('ENVIAYA_EXPRESS_FLAT_RATE_TITLE'),
                'ENVIAYA_EXPRESS_FLAT_RATE' => Tools::getValue('ENVIAYA_EXPRESS_FLAT_RATE'),
                'ENVIAYA_EXCLUDED_ZONE_REGIONS' => (Tools::getValue('ENVIAYA_EXCLUDED_ZONE_REGIONS') !== false) ? implode(',',Tools::getValue('ENVIAYA_EXCLUDED_ZONE_REGIONS')) : Tools::getValue('ENVIAYA_EXCLUDED_ZONE_REGIONS'),
                'ENVIAYA_EXCLUDED_ZONE_POSTCODES' => (Tools::getValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES') !== false) ? implode(',',explode("\n", str_replace("\r", "", Tools::getValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES')))) : Tools::getValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES'),
                'ENVIAYA_LOG' => Tools::getValue('ENVIAYA_LOG'),
                'ENVIAYA_COMPLETE_UNINSTALL' => Tools::getValue('ENVIAYA_COMPLETE_UNINSTALL'),
                'ENVIAYA_API_TIMEOUT' => Tools::getValue('ENVIAYA_API_TIMEOUT'),
            );

            foreach ($configuration_values as $configuration_key => $configuration_value) {
                Configuration::updateValue($configuration_key, $configuration_value);
            }
        }

        if (Tools::isSubmit('btnSubmitModal') ) {

            $configuration_values = array(
                Configuration::updateValue('ENVIAYA_LIVE_API_KEY', Tools::getValue('ENVIAYA_LIVE_API_KEY')),
                Configuration::updateValue('ENVIAYA_TEST_API_KEY', Tools::getValue('ENVIAYA_TEST_API_KEY')),
                Configuration::updateValue('ENVIAYA_TEST_MODE', Tools::getValue('ENVIAYA_TEST_MODE')),
                Configuration::updateValue('ENVIAYA_ACCOUNT', Tools::getValue('ENVIAYA_ACCOUNT')),
                Configuration::updateValue('ENVIAYA_ACCOUNT_ID', Tools::getValue('ENVIAYA_ACCOUNT_ID')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_ID', Tools::getValue('ENVIAYA_SENDER_ADDRESS_ID')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COMPANY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COMPANY')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME', Tools::getValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_PHONE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_PHONE')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_EMAIL', Tools::getValue('ENVIAYA_SENDER_ADDRESS_EMAIL')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1', Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2', Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD', Tools::getValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_CITY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_CITY')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_STATE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE')),
                Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COUNTRY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY')),
            );
            
            foreach ($configuration_values as $configuration_key => $configuration_value) {
                Configuration::updateValue($configuration_key, $configuration_value);
            }
        }
        
        $this->html = '';

        if (Tools::isSubmit('btnSubmit')) {
            
            Tools::clearSmartyCache();
            $this->postProcess();
        }
        
        //$this->html .= $this->displayCheck();
        $this->html .= $this->renderSettingsForm();
        
        return $this->html;
       
    }

    private function postProcess()
    {

        

        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('ENVIAYA_LIVE_API_KEY', Tools::getValue('ENVIAYA_LIVE_API_KEY'));
            Configuration::updateValue('ENVIAYA_TEST_API_KEY', Tools::getValue('ENVIAYA_TEST_API_KEY'));
            Configuration::updateValue('ENVIAYA_TEST_MODE', Tools::getValue('ENVIAYA_TEST_MODE'));
            Configuration::updateValue('ENVIAYA_ACCOUNT', Tools::getValue('ENVIAYA_ACCOUNT'));
            Configuration::updateValue('ENVIAYA_ACCOUNT_ID', Tools::getValue('ENVIAYA_ACCOUNT_ID'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_ID', Tools::getValue('ENVIAYA_SENDER_ADDRESS_ID'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COMPANY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COMPANY'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME', Tools::getValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_PHONE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_PHONE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_EMAIL', Tools::getValue('ENVIAYA_SENDER_ADDRESS_EMAIL'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1', Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2', Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD', Tools::getValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_CITY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_CITY'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_STATE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COUNTRY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY'));
            Configuration::updateValue('ENVIAYA_SHIPPING_CARRIER_NAME', Tools::getValue('ENVIAYA_SHIPPING_CARRIER_NAME'));
            Configuration::updateValue('ENVIAYA_SHIPPING_SERVICE_NAME', Tools::getValue('ENVIAYA_SHIPPING_SERVICE_NAME'));
            Configuration::updateValue('ENVIAYA_SHIPPING_DELIVERY_TIME', Tools::getValue('ENVIAYA_SHIPPING_DELIVERY_TIME'));
            Configuration::updateValue('ENVIAYA_DISPLAY_CARRIER_LOGO', Tools::getValue('ENVIAYA_DISPLAY_CARRIER_LOGO'));
            Configuration::updateValue('ENVIAYA_AS_DEFINED_PRICE', Tools::getValue('ENVIAYA_AS_DEFINED_PRICE'));
            Configuration::updateValue('ENVIAYA_ENABLE_CONTINGENCY_SHIPPING', Tools::getValue('ENVIAYA_ENABLE_CONTINGENCY_SHIPPING'));
            Configuration::updateValue('ENVIAYA_ENABLE_STANDARD_FLAT_RATE', Tools::getValue('ENVIAYA_ENABLE_STANDARD_FLAT_RATE'));
            Configuration::updateValue('ENVIAYA_STANDARD_FLAT_RATE_TITLE', Tools::getValue('ENVIAYA_STANDARD_FLAT_RATE_TITLE'));
            Configuration::updateValue('ENVIAYA_STANDARD_FLAT_RATE', Tools::getValue('ENVIAYA_STANDARD_FLAT_RATE'));
            Configuration::updateValue('ENVIAYA_ENABLE_EXPRESS_FLAT_RATE', Tools::getValue('ENVIAYA_ENABLE_EXPRESS_FLAT_RATE'));
            Configuration::updateValue('ENVIAYA_EXPRESS_FLAT_RATE_TITLE', Tools::getValue('ENVIAYA_EXPRESS_FLAT_RATE_TITLE'));
            Configuration::updateValue('ENVIAYA_EXPRESS_FLAT_RATE', Tools::getValue('ENVIAYA_EXPRESS_FLAT_RATE'));
            Configuration::updateValue('ENVIAYA_EXCLUDED_ZONE_REGIONS', (Tools::getValue('ENVIAYA_EXCLUDED_ZONE_REGIONS') !== false)  ? implode(',',Tools::getValue('ENVIAYA_EXCLUDED_ZONE_REGIONS')) : Tools::getValue('ENVIAYA_EXCLUDED_ZONE_REGIONS'));
            Configuration::updateValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES', (Tools::getValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES') !== false) ? implode(',',explode("\n", str_replace("\r", "", Tools::getValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES')))) : Tools::getValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES'));
            Configuration::updateValue('ENVIAYA_LOG', Tools::getValue('ENVIAYA_LOG'));
            Configuration::updateValue('ENVIAYA_COMPLETE_UNINSTALL', Tools::getValue('ENVIAYA_COMPLETE_UNINSTALL'));
            Configuration::updateValue('ENVIAYA_API_TIMEOUT', Tools::getValue('ENVIAYA_API_TIMEOUT'));

        }

        if (Tools::isSubmit('btnSubmitModal')) {
            Configuration::updateValue('ENVIAYA_LIVE_API_KEY', Tools::getValue('ENVIAYA_LIVE_API_KEY'));
            Configuration::updateValue('ENVIAYA_TEST_API_KEY', Tools::getValue('ENVIAYA_TEST_API_KEY'));
            Configuration::updateValue('ENVIAYA_TEST_MODE', Tools::getValue('ENVIAYA_TEST_MODE'));
            Configuration::updateValue('ENVIAYA_ACCOUNT', Tools::getValue('ENVIAYA_ACCOUNT'));
            Configuration::updateValue('ENVIAYA_ACCOUNT_ID', Tools::getValue('ENVIAYA_ACCOUNT_ID'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_ID', Tools::getValue('ENVIAYA_SENDER_ADDRESS_ID'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COMPANY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COMPANY'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME', Tools::getValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_PHONE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_PHONE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_EMAIL', Tools::getValue('ENVIAYA_SENDER_ADDRESS_EMAIL'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1', Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2', Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD', Tools::getValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_CITY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_CITY'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_STATE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE'));
            Configuration::updateValue('ENVIAYA_SENDER_ADDRESS_COUNTRY', Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY'));    

        }      

    }

    public function checkRequirements()
    {
        $tests = array(
            'result' => true,
            );

        $tests['curl'] = array(
            'name' => $this->l('php_curl_error'),
            'result' => (int) function_exists('curl_init'),
          );

        $tests['php52'] = array(
            'name' => $this->l('php_version_error'),
            'result' => (int) version_compare(PHP_VERSION, '5.6.0', '>='),
            );

        foreach ($tests as $k => $test) {
            if ('result' != $k && !$test['result']) {
                $tests['result'] = false;
            }
        }

        return $tests;
    }

    private function displayCheck()
    {
        //return $this->display(__FILE__, './views/templates/hook/infos.tpl');
    }

    public function renderSettingsForm()
    {   
       
        $configFields = $this->getConfigFieldsValues();
        
        // ACCESS FORM FIELDS
        $enviaya_account_options = array();
        if( (bool) $configFields['ENVIAYA_LIVE_API_KEY']  && (bool) $configFields['ENVIAYA_TEST_API_KEY'] ){
            
            $api_key = ((bool)$configFields['ENVIAYA_TEST_MODE']) ? $configFields['ENVIAYA_TEST_API_KEY'] : $configFields['ENVIAYA_LIVE_API_KEY'];
            
            Enviaya::setApiKey($api_key);

            $response =  Enviaya_Account::all(array(
                'api_key' => $api_key
            ));

            $query = array();
            foreach($response->__toArray(true)['enviaya_accounts'] as $key => $value){
                $query[] = array(
                    'value' => str_replace("\"","&quot;",Tools::jsonEncode($value)),
                    'text' => "{$value['alias']} ( {$value['account']} )"
                );
            }
            
            if(is_object($response)){
                $enviaya_account_options=array(
                    'query' => $query,
                    'id' => 'value',
                    'name' => 'text'
                );
                
            }
           
        }

        $access_out = array(
            array(
                'type' => 'text', 
                'label' => $this->l('api_key'),
                'name' => 'ENVIAYA_LIVE_API_KEY',
                'class' => 'form-control',
                'autocomplete' => false,
                'col' => '6',
                'required' => true,
                'desc' => $this->l('api_key_help').' <a href=https://app.enviaya.com.mx/api_keys target=_blank>'.$this->l('get_api_key').'</a>',
            ),
            array(
                'type' => 'text',
                'label' => $this->l('test_api_key'),
                'name' => 'ENVIAYA_TEST_API_KEY',
                'autocomplete' => false,
                'required' => true,
                'col' => '6',
            ),
            array(
                'type' => 'switch',
                'label' => $this->l('enable_test_mode'),
                'name' => 'ENVIAYA_TEST_MODE',
                'required' => true,
                'is_bool' => true,
                'desc' => $this->l('test_mode_help'),
                'values' => array(
                    
                    array(
                        'id' => 'test_mode_on',
                        'value' => 1,
                        'label' => $this->l('yes'),
                    ),
                    array(
                        'id' => 'test_mode_off',
                        'value' => 0,
                        'label' => $this->l('no'),
                    ),
                    
                ),
                
            ),
            array(
                'type' => 'select',
                'label' => $this->l('account'),
                'name' => 'ENVIAYA_ACCOUNT_SELECT',
                'required' => true,
                'col' => '6',
                'class' => '',
                'desc' => html_entity_decode(str_replace('/shipping/enviaya_accounts', '/accounts',$this->l('billing_account_help'))),
                'options' => $enviaya_account_options,
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_ACCOUNT',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_ACCOUNT_ID',
            ),
        );
        
        $access_fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('access_configuration'),
                    'icon' => '',
                    ),
                'input' => $access_out,
                'submit' => array(
                    'title' => $this->l('save'),
                ),
            ),

        );

        // SEMDER ADDRESS FORM FIELDS
        $enviaya_address_options = array();
        if((bool)$configFields['ENVIAYA_LIVE_API_KEY']  && (bool)$configFields['ENVIAYA_TEST_API_KEY'] ){
            
            $api_key = ((bool)$configFields['ENVIAYA_TEST_MODE']) ? $configFields['ENVIAYA_TEST_API_KEY'] : $configFields['ENVIAYA_LIVE_API_KEY'];
            
            Enviaya::setApiKey($api_key);

            $response =  Enviaya_Direction::all(array(
                'api_key' => $api_key,
                'get_destinations' => 'false',
            ));

            
            $query = array();
            foreach($response->__toArray(true)['directions'] as $key => $value){
                
                $query[] = array(
                    'value' => str_replace("\"","&quot;",Tools::jsonEncode($value)),
                    'text' => (!is_null($value['company'])) ? "{$value['full_name']} ( {$value['company']} )" : $value['full_name'],
                );
               
                if($value['id'] == $configFields['ENVIAYA_SENDER_ADDRESS_ID']){
                    $configFields['ENVIAYA_SENDER_ADDRESS'] =  Tools::jsonEncode($value);
                }
            }
            
            if(is_object($response)){
                $enviaya_address_options=array(
                    'query' => $query,
                    'id' => 'value',
                    'name' => 'text',                
                );
                
            }   
        }

        $sender_out = array(
            array(
                'type' => 'select',
                'label' => $this->l('sender_address'),
                'name' => 'ENVIAYA_SENDER_ADDRESS',
                'required' => true,
                'col' => '6',
                'desc' => '<a href=https://app.enviaya.com.mx/directions/new?origin=true target=_blank>'.$this->l('add_new_sender').'</a>',
                'options' => $enviaya_address_options,
            ),         
            array(
                'label' => $this->l('contact_information'),
                'type' => 'textarea',
                'name' => 'ENVIAYA_SENDER_CONTACT_INFORMATION',
                'col' => '6',
                'rows' => '3',
                'readonly' => true,
            ),
            array(
                'label' => $this->l('shipping_address'),
                'type' => 'textarea',
                'name' => 'ENVIAYA_SENDER_SHIPPING_ADDRESS',
                'col' => '6',
                'rows' => '5',
                'readonly' => true,
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_ID',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_FULL_NAME',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_COMPANY',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_PHONE',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_EMAIL',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_DIRECTION_1',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_DIRECTION_2',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_POSTAL_CODE',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_CITY',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_STATE',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_STATE_CODE',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_COUNTRY',
            ),
            array(
                'type' => 'hidden',
                'name' => 'ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE',
            ),
        );

        $sender_fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('sender_address'),
                    'icon' => '',
                ),
                'input' => $sender_out,
                'submit' => array(
                    'title' => $this->l('save'),
                ),
                
            ),

        );

        //RATING FORM FIELDS {#account_id#}
        $free_shipping_title_desc =  str_replace('/edit/{#account_id#}#api_configuration','',$this->l('free_shipping_text'));
        $additional_account_configurations_desc = preg_replace('/\/edit\/{\#account_id\#}\#\S*/', '', $this->l('additional_account_configurations_text'));
        
        if( (bool) $configFields['ENVIAYA_ACCOUNT']  && (bool) $configFields['ENVIAYA_ACCOUNT_ID'] )
        {
            $free_shipping_title_desc =  str_replace('/accounts','/accounts/edit/'.$configFields['ENVIAYA_ACCOUNT_ID'].'#api_configuration',$free_shipping_title_desc);
            $additional_account_configurations_desc = str_replace('{#account_id#}',$configFields['ENVIAYA_ACCOUNT_ID'], $this->l('additional_account_configurations_text'));
        }

        $rating_out = array(
            array(
                'type' => 'free',
                'name' => 'shipping_services_display_title',

                
            ),
            array(
                'type' => 'select',
                'label' => $this->l('carrier_name'),
                'name' => 'ENVIAYA_SHIPPING_CARRIER_NAME',
                'required' => true,
                'col' => '6',
                'desc' => $this->l('carrier_name_help'),
                'options' => array(
                    'query' => array(
                        array(
                            'value' => 0,
                            'text' => $this->l('carrier_name_option_1'),
                        ), 
                        array(
                            'value' => 1,
                            'text' => $this->l('carrier_name_option_2'),
                        ),      
                    ),
                    'id' => 'value',
                    'name' => 'text'
                ),
            ),
            array(
                'type' => 'select',
                'label' => $this->l('service_name'),
                'name' => 'ENVIAYA_SHIPPING_SERVICE_NAME',
                'required' => true,
                'col' => '6',
                'desc' => html_entity_decode($this->l('service_name_help')),
                'options' => array(
                    'query' => array(
                        array(
                            'value' => 1,
                            'text' => $this->l('service_name_1'),
                        ), 
                        array(
                            'value' => 3,
                            'text' => $this->l('service_name_3'),
                        ),
                        array(
                            'value' => 4,
                            'text' => $this->l('service_name_4'),
                        ),              
                    ),
                    'id' => 'value',
                    'name' => 'text'
                ),
            ),
            array(
                'type' => 'select',
                'label' => $this->l('shipping_delivery_time'),
                'name' => 'ENVIAYA_SHIPPING_DELIVERY_TIME',
                'required' => true,
                'col' => '6',
                'desc' => html_entity_decode($this->l('shipping_delivery_time_help')),
                'options' => array(
                    'query' => array(
                        array(
                            'value' => 1,
                            'text' => $this->l('show_estimated_delivery_date'),
                        ),
                        array(
                            'value' => 2,
                            'text' => $this->l('show_transit_time'),
                        ),
                        array(
                            'value' => 3,
                            'text' => $this->l('do_not_show_delivery_time'),
                        ),              
                    ),
                    'id' => 'value',
                    'name' => 'text'
                ),
            ),
            array(
                'type' => 'select',
                'label' => $this->l('display_carrier_logo'),
                'name' => 'ENVIAYA_DISPLAY_CARRIER_LOGO',
                'required' => true,
                'col' => '6',
                'options' => array(
                    'query' => array(
                        array(
                            'value' => 1,
                            'text' => $this->l('yes'),
                        ),
                        array(
                            'value' => 2,
                            'text' => $this->l('no'),
                        ),          
                    ),
                    'id' => 'value',
                    'name' => 'text'
                ),
            ),
            array(
                'type' => 'free',
                'name' => 'free_shipping_title',
                
                'hint' =>  html_entity_decode($free_shipping_title_desc),                
            ),
            array(
                'type' => 'select',
                'label' => $this->l('as_defined_price'),
                'name' => 'ENVIAYA_AS_DEFINED_PRICE',
                'required' => true,
                'col' => '6',
                'options' => array(
                    'query' => array(
                        array(
                            'value' => 1,
                            'text' => $this->l('show_price_as'),
                        ),
                        array(
                            'value' => 0,
                            'text' => $this->l('do_not_show_price'),
                        ),          
                    ),
                    'id' => 'value',
                    'name' => 'text'
                ),
            ),
            array(
                'type' => 'free',
                'name' => 'contingency_shipping_title',
                'hint' =>  html_entity_decode($this->l('contingency_services_help')),                
            ),
            array(
                'type' => 'select',
                'label' => $this->l('enable_contingency_shipping_services'),
                'name' => 'ENVIAYA_ENABLE_CONTINGENCY_SHIPPING',
                'required' => true,
                'col' => '6',
                'options' => array(
                    'query' => array(
                        array(
                            'value' => 1,
                            'text' => $this->l('yes'),
                        ),
                        array(
                            'value' => 0,
                            'text' => $this->l('no'),
                        ),          
                    ),
                    'id' => 'value',
                    'name' => 'text'
                ),
                
            ),
            array(
                'type' => 'switch',
                'label' => html_entity_decode(str_replace('(','<br>(',$this->l('contingency_standard_shipping'))),
                'name' => 'ENVIAYA_ENABLE_STANDARD_FLAT_RATE',
                'required' => true,
                'is_bool' => true,
                'values' => array(
                    array(
                        'id' => 'standard_flate_rate_on',
                        'value' => 1,
                        'label' => $this->l('yes'),
                    ),
                    array(
                        'id' => 'standard_flate_rate_off',
                        'value' => 0,
                        'label' => $this->l('no'),
                    ),  
                ),
            ),
            array(
                'type' => 'text', 
                'label' => $this->l('contingency_standard_shipping_title'),
                'name' => 'ENVIAYA_STANDARD_FLAT_RATE_TITLE',
                'class' => 'form-control',
                'col' => '6',
                'required' => false,
                'desc' => $this->l('contingency_standard_shipping_help')
            ),
            array(
                'type' => 'text', 
                'label' => $this->l('contingency_standard_rate'),
                'name' => 'ENVIAYA_STANDARD_FLAT_RATE',
                'class' => 'form-control',
                'col' => '6',
                'required' => false,
                'desc' => $this->l('contingency_standard_rate_help')
            ),
            array(
                'type' => 'switch',
                'label' => html_entity_decode(str_replace('(','<br>(',$this->l('contingency_express_shipping'))),
                'name' => 'ENVIAYA_ENABLE_EXPRESS_FLAT_RATE',
                'required' => true,
                'is_bool' => true,
                'values' => array(
                    array(
                        'id' => 'express_flate_rate_on',
                        'value' => 1,
                        'label' => $this->l('yes'),
                    ),
                    array(
                        'id' => 'express_flate_rate_off',
                        'value' => 0,
                        'label' => $this->l('no'),
                    ),  
                ),
            ),
            array(
                'type' => 'text', 
                'label' => $this->l('contingency_express_shipping_title'),
                'name' => 'ENVIAYA_EXPRESS_FLAT_RATE_TITLE',
                'class' => 'form-control',
                'col' => '6',
                'required' => false,
                'desc' => $this->l('contingency_express_shipping_help')
            ),
            array(
                'type' => 'text', 
                'label' => $this->l('contingency_express_rate'),
                'name' => 'ENVIAYA_EXPRESS_FLAT_RATE',
                'class' => 'form-control',
                'col' => '6',
                'required' => false,
                'desc' => $this->l('contingency_express_rate_help')
            ),
            array(
                'type' => 'free',
                'name' => 'additional_account_configurations_title',
                'hint' =>  html_entity_decode($additional_account_configurations_desc),                
            ),
            array(
                'type' => 'free',
                'name' => 'carrier_and_services_configuration_title',
                'hint' =>  html_entity_decode($this->l('carrier_and_services_configuration_help')),                
            ),
            array(
                'type' => 'free',
                'name' => 'subsidies_title',
                'hint' =>  array(
                    html_entity_decode($this->l('subsidies_text')),
                    html_entity_decode($this->l('subsidies_link_text')), 
                )            
            ),
            
            
            
        );


        $rating_fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('rating_configuration'),
                    'icon' => '',
                ),
                'input' => $rating_out,
                'submit' => array(
                    'title' => $this->l('save'),
                ),
                
            ),

        );

        $excluded_zone_out = array(

            array(
                'type' => 'free',
                'name' => 'excluded_zones_title',
                'hint' => html_entity_decode(explode('(',$this->l('excluded_zones_title_help'))[0])          
            ),
            array(
                'type' => 'select',
                'label' => $this->l('zone_regions'),
                'name' => 'ENVIAYA_EXCLUDED_ZONE_REGIONS', 
                'required' => false,
                'col' => '6',
                'desc' => $this->l('zone_regions_help'),
                'multiple' => true,
                'options' => array(
                    'query' => $this->getStates(Context::getContext()->language->id),
                    'id' => 'value',
                    'name' => 'text'
                ),
            ),
            array(
                'type' => 'free',
                'value' => $configFields['ENVIAYA_EXCLUDED_ZONE_POSTCODES'],
                'label' => '',
                'col' => 6,
                'name' => 'ENVIAYA_EXCLUDED_ZONE_POSTCODES', 
                'desc' => html_entity_decode(explode(htmlspecialchars('99000</code>) '),$this->l('zone_regions_list_help'))[0].htmlspecialchars('99000</code>)')),
            ),

        );

        $excluded_fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('excluded_zones'),
                    'icon' => '',
                ),
                'input' => $excluded_zone_out,
                'submit' => array(
                    'title' => $this->l('save'),
                ),
                
            ),

        );


        $logs_url =  $this->context->link->getAdminLink('AdminLogs', false).'&logs[filters][message]=EnviaYa&token='. Tools::getAdminTokenLite('AdminLogs');

        $technique_out = array(
            array(
                'type' => 'text', 
                'label' => $this->l('timeout'),
                'name' => 'ENVIAYA_API_TIMEOUT',
                'class' => 'form-control',
                'col' => '6',
                'required' => false,
                'desc' => $this->l('timeout_help')
            ),
            array(
                'type' => 'switch', 
                'label' => $this->l('enable_api_logs'),
                'name' => 'ENVIAYA_LOG',
                'is_bool' => true,
                'desc' => $this->l('enable_api_logs_help').' <a href="'.$logs_url.'"> '.$this->l('logger_api_action').'</a>',
                'values' => array(
                
                    array(
                        'id' => 'api_logs_yes',
                        'value' => 1,
                        'label' => $this->l('yes'),
                    ),
                    array(
                        'id' => 'api_logs_no',
                        'value' => 0,
                        'label' => $this->l('no'),
                    ),
                    
                ),
            ),
            array(
                'type' => 'select',
                'label' => $this->l('remove_all_settings_and_data'),
                'name' => 'ENVIAYA_COMPLETE_UNINSTALL',
                'required' => true,
                'col' => '6',
                'desc' => $this->l('remove_all_settings_and_data_help'), 
                'options' => array(
                    'query' => array(
                        array(
                            'value' => 1,
                            'text' => $this->l('yes'),
                        ),
                        array(
                            'value' => 0,
                            'text' => $this->l('no'),
                        ),          
                    ),
                    'id' => 'value',
                    'name' => 'text'
                ),
                
            ),
        );

        $technique_fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('technical_configuration'),
                    'icon' => '',
                ),
                'input' => $technique_out,
                'submit' => array(
                    'title' => $this->l('save'),
                ),
                
            ),

        );

        $requirements = $this->checkRequirements();


        $status_out = array(
            array(
                'type' => 'free',
                'name' => 'status_title',
                'valie' => $requirements,
            ),
            array(
                'type' => 'free',
                'label' => $this->l('domain'),
                'col' => 6,
                'name' => 'ENVIAYA_DOMAIN', 
            ),
            array(
                'type' => 'free',
                'label' => $this->l('php_version'),
                'col' => 6,
                'name' => 'ENVIAYA_PHP_VERSION' 
            ),
            array(
                'type' => 'free',
                'label' => $this->l('presta_version'),
                'col' => 6,
                'name' => 'ENVIAYA_PS_VERSION' 
            ),
            array(
                'type' => 'free',
                'label' => $this->l('curl_existence'),
                'col' => 6,
                'name' => 'ENVIAYA_CURL', 
                'value' =>  ( $requirements['curl']['result'] ) ? $this->l('yes') : $this->l('no')
            )

            

            
        );
    
        $status_fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('status'),
                    'icon' => '',
                ),
                'input' => $status_out,
                
            ),

        );
       

        // set up form
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->title = $this->displayName;
        $default_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        //$helper->submit_action = 'submit'.$this->name;
        $helper->submit_action = 'btnSubmit';
        $helper->toolbar_btn = array(
            'save' => array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'desc' => $this->l('Back to List'),
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
        );
     
        if(isset($configFields['ENVIAYA_EXCLUDED_ZONE_REGIONS'])){
            $key = 'ENVIAYA_EXCLUDED_ZONE_REGIONS[]';
            $configFields[$key] = $configFields['ENVIAYA_EXCLUDED_ZONE_REGIONS'];
            unset($configFields['ENVIAYA_EXCLUDED_ZONE_REGIONS']);
        }

       
        $helper->tpl_vars = array(
            'fields_value' => $configFields,
            'id_language' => $this->context->language->id,
            'languages' => $this->context->controller->getLanguages(),
            'back_url' => $this->context->link->getAdminLink('AdminModules')
                .'&configure='.$this->name
                .'&tab_module='.$this->tab
                .'&module_name='.$this->name
                .'#general',
            //'color' => true,
        );

        //generate form with settings fields

        return $helper->generateForm(array($access_fields_form,$rating_fields_form,$excluded_fields_form,$sender_fields_form, $technique_fields_form,$status_fields_form));
        if (count($this->_errors)) {
            //$this->message .= $this->displayError($this->_errors);
        }
        

    }

    public function getConfigFieldsValues()
    {


        return array(

          'ENVIAYA_LIVE_API_KEY' => Tools::getValue('ENVIAYA_LIVE_API_KEY', Configuration::get('ENVIAYA_LIVE_API_KEY')),
          'ENVIAYA_TEST_API_KEY' => Tools::getValue('ENVIAYA_TEST_API_KEY', Configuration::get('ENVIAYA_TEST_API_KEY')),
          'ENVIAYA_TEST_MODE' => Tools::getValue('ENVIAYA_TEST_MODE', Configuration::get('ENVIAYA_TEST_MODE')),
          'ENVIAYA_ACCOUNT' => Tools::getValue('ENVIAYA_ACCOUNT', Configuration::get('ENVIAYA_ACCOUNT')),
          'ENVIAYA_ACCOUNT_ID' => Tools::getValue('ENVIAYA_ACCOUNT_ID', Configuration::get('ENVIAYA_ACCOUNT_ID')),
          'ENVIAYA_SENDER_ADDRESS_ID' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_ID', Configuration::get('ENVIAYA_SENDER_ADDRESS_ID')),
          'ENVIAYA_SENDER_ADDRESS_COMPANY' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_COMPANY', Configuration::get('ENVIAYA_SENDER_ADDRESS_COMPANY')),
          'ENVIAYA_SENDER_ADDRESS_FULL_NAME' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_FULL_NAME', Configuration::get('ENVIAYA_SENDER_ADDRESS_FULL_NAME')),
          'ENVIAYA_SENDER_ADDRESS_POSTAL_CODE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE', Configuration::get('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE')),
          'ENVIAYA_SENDER_ADDRESS_CITY' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_CITY', Configuration::get('ENVIAYA_SENDER_ADDRESS_CITY')),
          'ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD', Configuration::get('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD')),
          'ENVIAYA_SENDER_ADDRESS_DIRECTION_1' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_1', Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_1')),
          'ENVIAYA_SENDER_ADDRESS_DIRECTION_2' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_DIRECTION_2', Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_2')),
          'ENVIAYA_SENDER_ADDRESS_PHONE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_PHONE', Configuration::get('ENVIAYA_SENDER_ADDRESS_PHONE')),
          'ENVIAYA_SENDER_ADDRESS_EMAIL' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_EMAIL', Configuration::get('ENVIAYA_SENDER_ADDRESS_EMAIL')),
          'ENVIAYA_SENDER_ADDRESS_STATE_CODE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE_CODE', Configuration::get('ENVIAYA_SENDER_ADDRESS_STATE_CODE')),
          'ENVIAYA_SENDER_ADDRESS_STATE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_STATE', Configuration::get('ENVIAYA_SENDER_ADDRESS_STATE')),
          'ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE', Configuration::get('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE')),
          'ENVIAYA_SENDER_ADDRESS_COUNTRY' => Tools::getValue('ENVIAYA_SENDER_ADDRESS_COUNTRY', Configuration::get('ENVIAYA_SENDER_ADDRESS_COUNTRY')),
          'ENVIAYA_SHIPPING_CARRIER_NAME' => Tools::getValue('ENVIAYA_SHIPPING_CARRIER_NAME', Configuration::get('ENVIAYA_SHIPPING_CARRIER_NAME')),   
          'ENVIAYA_SHIPPING_SERVICE_NAME' => Tools::getValue('ENVIAYA_SHIPPING_SERVICE_NAME', Configuration::get('ENVIAYA_SHIPPING_SERVICE_NAME')), 
          'ENVIAYA_SHIPPING_DELIVERY_TIME' => Tools::getValue('ENVIAYA_SHIPPING_DELIVERY_TIME', Configuration::get('ENVIAYA_SHIPPING_DELIVERY_TIME')), 
          'ENVIAYA_DISPLAY_CARRIER_LOGO' => Tools::getValue('ENVIAYA_DISPLAY_CARRIER_LOGO', Configuration::get('ENVIAYA_DISPLAY_CARRIER_LOGO')), 
          'ENVIAYA_AS_DEFINED_PRICE' => Tools::getValue('ENVIAYA_AS_DEFINED_PRICE', Configuration::get('ENVIAYA_AS_DEFINED_PRICE')), 
          'ENVIAYA_ENABLE_CONTINGENCY_SHIPPING' => Tools::getValue('ENVIAYA_ENABLE_CONTINGENCY_SHIPPING', Configuration::get('ENVIAYA_ENABLE_CONTINGENCY_SHIPPING')), 
          'ENVIAYA_ENABLE_STANDARD_FLAT_RATE' => Tools::getValue('ENVIAYA_ENABLE_STANDARD_FLAT_RATE', Configuration::get('ENVIAYA_ENABLE_STANDARD_FLAT_RATE')), 
          'ENVIAYA_STANDARD_FLAT_RATE_TITLE' => Tools::getValue('ENVIAYA_STANDARD_FLAT_RATE_TITLE', Configuration::get('ENVIAYA_STANDARD_FLAT_RATE_TITLE')), 
          'ENVIAYA_STANDARD_FLAT_RATE' => Tools::getValue('ENVIAYA_STANDARD_FLAT_RATE', Configuration::get('ENVIAYA_STANDARD_FLAT_RATE')), 
          'ENVIAYA_ENABLE_EXPRESS_FLAT_RATE' => Tools::getValue('ENVIAYA_ENABLE_EXPRESS_FLAT_RATE', Configuration::get('ENVIAYA_ENABLE_EXPRESS_FLAT_RATE')), 
          'ENVIAYA_EXPRESS_FLAT_RATE_TITLE' => Tools::getValue('ENVIAYA_EXPRESS_FLAT_RATE_TITLE', Configuration::get('ENVIAYA_EXPRESS_FLAT_RATE_TITLE')), 
          'ENVIAYA_EXPRESS_FLAT_RATE' => Tools::getValue('ENVIAYA_EXPRESS_FLAT_RATE', Configuration::get('ENVIAYA_EXPRESS_FLAT_RATE')), 
          'ENVIAYA_EXCLUDED_ZONE_REGIONS' => Tools::getValue('ENVIAYA_EXCLUDED_ZONE_REGIONS', Configuration::get('ENVIAYA_EXCLUDED_ZONE_REGIONS')), 
          'ENVIAYA_EXCLUDED_ZONE_POSTCODES' => Tools::getValue('ENVIAYA_EXCLUDED_ZONE_POSTCODES', Configuration::get('ENVIAYA_EXCLUDED_ZONE_POSTCODES')), 
          'ENVIAYA_API_TIMEOUT' => Tools::getValue('ENVIAYA_API_TIMEOUT', Configuration::get('ENVIAYA_API_TIMEOUT')), 
          'ENVIAYA_COMPLETE_UNINSTALL' => Tools::getValue('ENVIAYA_COMPLETE_UNINSTALL', Configuration::get('ENVIAYA_COMPLETE_UNINSTALL')), 
          'ENVIAYA_LOG' => Tools::getValue('ENVIAYA_LOG', Configuration::get('ENVIAYA_LOG')),

          'ENVIAYA_ACCOUNT_SELECT'=> Tools::getValue('ENVIAYA_ACCOUNT_SELECT'),
          'shipping_services_display_title'=> Tools::getValue('shipping_services_display_title'),
          'free_shipping_title'=> Tools::getValue('free_shipping_title'),
          'contingency_shipping_title'=> Tools::getValue('contingency_shipping_title'),
          'additional_account_configurations_title'=> Tools::getValue('additional_account_configurations_title'),
          'carrier_and_services_configuration_title'=> Tools::getValue('carrier_and_services_configuration_title'),
          'subsidies_title'=> Tools::getValue('subsidies_title'),
          'excluded_zones_title'=> Tools::getValue('excluded_zones_title'),
          'ENVIAYA_SENDER_CONTACT_INFORMATION'=> Tools::getValue('ENVIAYA_SENDER_CONTACT_INFORMATION'),
          'ENVIAYA_SENDER_SHIPPING_ADDRESS'=> Tools::getValue('ENVIAYA_SENDER_SHIPPING_ADDRESS'),
          'status_title'=> Tools::getValue('status_title'),


        );


    }

    public function hookDisplayAdminOrderTabContent($params)
    {
        
      // Since Ps 1.7.7 this hook is displayed at bottom of a page and we should use a hook DisplayAdminOrderTop
        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            
            $order = new Order((int)$params['id_order']);
            
            $params['order'] = $order;
            //return $this->hookDisplayAdminOrderContentOrder($params);
            
            return $this->hookDisplayOrderDetail($order);
        }
    }

    public function hookDisplayBackOfficeHeader($params)
    {
        
        $token = Tools::getAdminTokenLite('AdminModules');
            //$ajax_url_module = 'index.php?tab=AdminModules&configure='.$this->name.'&token='.$token;
            $ajax_url_module = __PS_BASE_URI__.basename(_PS_ADMIN_DIR_).'/index.php?tab=AdminModules&configure='.$this->name.'&token='.$token;
            Media::addJsDef(array(
                'ajax_url_module' => $ajax_url_module,
                //'ajax_url_enviayacarrier_module' => $ajax_url_enviayacarrier_module,
            ));
        
        if (Tools::getValue('configure') == $this->name) {

            $this->context->controller->addJquery();
            $this->context->controller->addJS(array($this->_path.'views/js/adminscript-enviayacarrier.js'));
            $this->context->controller->addJS(array($this->_path.'views/lib/select2-4.0.13/js/select2.min.js'));

            $this->context->controller->addCSS($this->_path.'views/lib/select2-4.0.13/css/select2.min.css');
        }
        if (Tools::getValue('controller') == 'AdminOrders') {
           
          

            $this->context->controller->addJquery();
            $this->context->controller->addJS(array($this->_path.'views/js/adminscript-enviayacarrier.js'));
        }
        $this->context->controller->addCSS($this->_path.'views/css/admin.css');
    }

    public function hookDisplayHeader($params)
    {
        
        if (Module::isEnabled('enviayacarrier')) {
            $cart = isset($params['cart']) ? $params['cart'] : null;
            $controller = isset($this->context->controller) ?
            $this->context->controller : null;

            $allowed_controllers = array(
            'HistoryController',
            'OrderConfirmationController',
            'OrderController',
            'OrderOpcController',
            );

            $is_allowed = !is_null($controller) &&
            in_array(get_class($controller), $allowed_controllers);

            if (!$is_allowed || !$cart) {
                // Load assets just in the order-related controllers.
            //return '';
            }

            /*if (!$this->serviceRatesAvailable()) {
                return '';
            }*/
            $ajax_url = Context::getContext()->link->getModuleLink('enviayacarrier', 'servicerateselection', array('idPayment' => 1337));
            $onepagecheckout = false;
            if (Module::isEnabled('onepagecheckoutps')) {
                $onepagecheckout = true;
            }

            //find installed our carrier id
            $carrier = new Carrier($this->getEnviayaCarrier());

            $enviaya_carrier_id = (int) $carrier->id_reference;
            Media::addJsDef(array('ajax_url' => $ajax_url));
            Media::addJsDef(array('onepagecheckout' => $onepagecheckout));
            Media::addJsDef(array('enviaya_carrier_id' => $enviaya_carrier_id));
            //$script = $this->connector->getServiceRateScript();
            $script = 'frontscript-enviayacarrier.js';

            if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
                $controller->registerStylesheet(
                    'module-enviaya-frontstyles',
                    'modules/'.$this->name.'/views/css/front.css',
                    array('media' => 'screen')
                );
                $this->context->controller->addJS(array($this->_path.'views/js/'.$script));
            } else {
                $controller->addCSS($this->_path.'/views/css/front.css');
                $controller->addJquery();
                $controller->addJS($script, false);
            }
        }
    }
    
    /**
     * Display the service rate details in the order details tab (history)
     * to the customer.
     *
     * @param array $params
     *
     * @return string
     */
    public function hookDisplayOrderDetail($params)
    {
        
        
        $order_id = isset($params->id) ? $params->id : null;
        if ($order_id) {
            
            require_once dirname(__FILE__).'/api/loader.php';
           
            $carrier = new Carrier((int) $params->id_carrier);
           
            $idCarrierReference = $carrier->id_reference;

            $service_rate = EnviayaServiceRate::getFromCart($params->id_cart);
           
            if (!$service_rate->id) {
                //skip
            } else {
                
                $response = unserialize($service_rate->getDetails());
               
                
                $response_selected = unserialize($service_rate->getSelected());
                
                $booking = unserialize($service_rate->getBooking()); 
                $request = $service_rate->getRequestsaved();
       
                if (!empty($booking)) {
                    //throw new Exception(json_encode($booking->__toArray(true)));
                    if (is_object($booking) and empty($booking['erros']) ) {
                        if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                            PrestaShopLogger::addLog('[EnviaYa] return shipment #'.print_r($booking, true), 1);
                        }
                        
                        //booking success send array to smarty
                        $this->smarty->assign(
                            array('booking' => $booking)
                        );
                        
                        
                        if(empty($service_rate->extra_data)){
                            
                            Enviaya::setApiKey($booking->getApiKey());
                            $response = Enviaya_Track::create(array(
                                'api_key' => $booking->getApiKey(),
                                'carrier' => $booking->rate['carrier'],
                                'shipment_number' => $booking->enviaya_shipment_number,
                            ));

                            if (is_object($response))  {

                
                                $response->__set('last_update', date('Y-m-dTH:i:s') );
                                $serializedTracking = serialize($response);
                              
                                $service_rate->extra_data = Tools::jsonEncode($serializedTracking);
                                $service_rate->save();

                                
                            }

                        }
                        
                        $tracking = unserialize($service_rate->geTracking());
                        $this->smarty->assign(
                            array(
                                'tracking' => $tracking,
                                'last_update' => date( Context::getContext()->language->date_format_full, strtotime($tracking->last_update)  )
                            )
                        );
                        
    
                    } else {
                        $this->smarty->assign(
                            array('booking' => (string) Tools::jsonEncode($booking),
                             )
                        );
                    }
                } else {
                    
                    if (is_object($response) && empty($response['errors'])){
                        
                        foreach($response->rates as $rate){
                           
                            $estimated_delivery = 'N / A';
                            if( !is_null($rate->estimated_delivery) ){
                                $estimated_delivery = date( Context::getContext()->language->date_format_lite, strtotime($rate->estimated_delivery)  );
                            }
                            $rate->__set('estimated_delivery', $estimated_delivery);
                           
                           
                        }
                        $warnings = array();
                        if(isset($response['warning'])){
                            $warnings = $response['warning'];
                        }
                        
                        $this->smarty->assign(
                            array(
                                'warnings' => $warnings,
                                'rates' => $response->rates,
                                'rate_selected' => $response_selected,
                                'request' => $request,
                            )
                        );

                    }else{
                        
                        
                        $errors[] = $this->l('rating_error_no_services_found');

                        if(in_array($response_selected->rate_id, array('enviaya_standard_flat_rate','enviaya_express_flat_rate'))){
                            $errors[] = $this->l('contingency_shiiping_service_enabled');
                            $errors[] = sprintf('<b>%s</b>',$response_selected->carrier_service_name.'<b> ( '.$response_selected->total_amount.'  '.$response_selected->currency.' )</b>');
                        }
                      
                        $this->smarty->assign(
                            array(
                                'errors' => $errors,
                                'rates' => array(),
                                'rate_selected' => $response_selected,
                            
                            )
                        );
                    }

                }
 

            }

                   
            $this->smarty->assign(
                array(
                'enviaya_logo' => _THEME_SHIP_DIR_.(int) $params->id_carrier.'.jpg',
                'order_id' => $order_id
                )
            );
            //always render to allow Export
            return $this->display(__FILE__, 'views/templates/hook/order-details.tpl');
            
           
        }

    }

    /**
     * Display the *contents* of the tab containinig the service point details
     * in the Admin order editing view.
     *
     * @param array $params
     *
     * @return string
     */
    public function hookDisplayAdminOrderContentShip(array $params)
    {
        
        $order = isset($params['order']) ? $params['order'] : null;
        
        //return $this->displayAdminOrderServicePoint($order);
        return $this->hookDisplayOrderDetail($order);

    }

    public function hookDisplayCarrierExtraContent($params){
        return $this->displayServiceRateButton($params);
    }

    private function displayServiceRateButton(array $params)
    {
   
        $cart = isset($params['cart']) ? $params['cart'] : null;
        if (!$cart || !$cart->id_address_delivery) {
            return false;
        }

        require_once dirname(__FILE__).'/api/loader.php';
        
        $service_rate = EnviayaServiceRate::getFromCart($cart->id);

        $ratesreceived = unserialize(Tools::jsonDecode($service_rate->details, true));
        $ratesreceived_extra = array();
        $ratesreceived_extra['rates'] = array();

        if (!empty($ratesreceived['rates'])) {
            
            $configFieldsValues = $this->getConfigFieldsValues();

            //ORDER 
           
            foreach ($ratesreceived['rates'] as $quotekey => $qtvalue) {


                if ($this->iSobjctKeyExists('currency', $qtvalue)) {
                    if (Currency::exists($qtvalue->currency)) {

                        //$EnviayaCurrency = new Currency();
                        $EnviayaCurrencyId = Currency::getIdByIsoCode($qtvalue->currency);
                        //$ratesreceived['rates'][$quotekey]['testrate'] = 'aaa';
                        //$ratesreceived['rates'][$quotekey]->testrate = 'bbb';
                        //will not work indirect modification of object
                        //$ratesreceived['rates'][$quotekey]['currency_id'] = $EnviayaCurrencyId;
                        //save to extra
                        $ratesreceived_extra['rates'][$quotekey]['currency_id'] = $EnviayaCurrencyId;

                        $EnviayaCurrencyData = new Currency((int) $EnviayaCurrencyId);
                        $customerCurrency = new Currency((int) $cart->id_currency);

                        // We need to convert the price in the default currency of the shop
                        $price = $qtvalue->total_amount / $EnviayaCurrencyData->conversion_rate;
                        // Then we convert it in the currency of the cart
                        $price = $price * $customerCurrency->conversion_rate;

                        if(isset($qtvalue->additional_configuration->free_shipping)){
                            $price = 0;
                        }

                        $totalCartWithTaxes = $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS);
                        $totalCartWithTaxes = $totalCartWithTaxes / $customerCurrency->conversion_rate;

                        if(isset($ratesreceived['subsidy']) && $totalCartWithTaxes >= $ratesreceived['subsidy']['minimum_order_amount']){
                            $price = max($price - $ratesreceived['subsidy']['total_amount'],0);
                        }

                        //$ratesreceived['rates'][$quotekey]->amountInCartCurrency = $this->getRounddNumber($price);

                        $service_name = ( $configFieldsValues['ENVIAYA_SHIPPING_SERVICE_NAME'] == "1" && !is_null($qtvalue->dynamic_service_name) ) ?  $qtvalue->dynamic_service_name : $qtvalue->carrier_service_name;
                        $delivery_time =  'N/D';

                        if( isset($qtvalue->estimated_delivery) && $configFieldsValues['ENVIAYA_SHIPPING_DELIVERY_TIME'] == "1" ){
                            $delivery_time = date( Context::getContext()->language->date_format_lite, strtotime($qtvalue->estimated_delivery)  );
                        }

                        if( isset($qtvalue->est_transit_time_hours) && $qtvalue->est_transit_time_hours > 0 && $configFieldsValues['ENVIAYA_SHIPPING_DELIVERY_TIME'] == "2" ){
                            $delivery_time = $this->calcDeliveryTime($qtvalue->est_transit_time_hours);
                        }

                        $ratesreceived_extra['rates'][$quotekey]['rate_id'] = $qtvalue->rate_id;
                        $ratesreceived_extra['rates'][$quotekey]['service_name'] = $service_name;
                        $ratesreceived_extra['rates'][$quotekey]['carrier_name'] = $qtvalue->carrier;
                        $ratesreceived_extra['rates'][$quotekey]['amountInCartCurrency'] = $this->getRounddNumber($price);
                        $ratesreceived_extra['rates'][$quotekey]['original_object_rates_id'] = $quotekey;
                        $ratesreceived_extra['rates'][$quotekey]['image'] = $qtvalue->carrier_logo_url;
                        $ratesreceived_extra['rates'][$quotekey]['free_shipping'] = ( isset($qtvalue->additional_configuration->free_shipping) ) ? $qtvalue->additional_configuration->free_shipping : null;
                        $ratesreceived_extra['rates'][$quotekey]['store_pickup'] = ( (string)$quotekey == 'store_pickup' ) ? true : false;
                        $ratesreceived_extra['rates'][$quotekey]['delivery_time'] = $delivery_time;
                       

                    }
                }
            }

            //selected
            $selected_carrier = unserialize(Tools::jsonDecode($service_rate->selected, true));
        }
     
        if (!empty($ratesreceived['rates'])) {

        
            //$carrier = $this->connector->getOrSynchroniseCarrier();
            $carrier = new Carrier((int) $params['cart']->id_carrier);
            $idCarrierReference = $carrier->id_reference;
            $address = new Address($cart->id_address_delivery);
            $country = new Country($address->id_country);
            //$token = sha1('');
            $link = $this->context->link;
            $this->smarty->assign(array(
            'config_fields_values' => $configFieldsValues,
            'prestashop_version' => _PS_VERSION_,
            'carrier' => $carrier,
            'cart' => $cart,
            'to_country' => $country->iso_code,
            'to_postal_code' => $address->postcode,
            'language' => $this->context->language->language_code,
            'service_rate_details' => $ratesreceived,
            'selected_carrier' => $selected_carrier,
            'ratesreceived_extra' => $ratesreceived_extra,
            'token' => Tools::getToken(false),
            'save_endpoint' => $link->getModuleLink($this->name, 'ServiceRateSelection'),
            ));

            return $this->display(
                __FILE__,
                'views/templates/hook/carrier-selection.tpl'
            );
        }
        


    }

    private function saveServiceRate(Cart $cart, $raw_data, $hash, $requestsaved)
    {
        //$details = urldecode($raw_data);
        $details = $raw_data;
        
        if (!Tools::jsonDecode($details) || !Tools::jsonDecode($requestsaved)) {
            return false;
        }

        $service_rate = EnviayaServiceRate::getFromCart($cart->id);
        
        $service_rate->details = $details;
        $service_rate->hash = $hash;
        $service_rate->requestsaved = $requestsaved;
        
        //to prevent old selected data to be stored compate is selected data exist in the details and if not clean up selected
        $selected = $service_rate->getSelected(); //$service_rate->selected;

        if ($this->iSarrayKeyExists('carrier', $selected)) {
            //saved carrier exist start to compate with latest details
            //search carrier id
            //$key = array_search($selected['servicelevel']->token, array_column($details, 'carrier'));
            $found = false;
            $detailsarray = Tools::jsonDecode($details, true);
            if (is_array($detailsarray) and $this->iSarrayKeyExists('quotes', $detailsarray) and $this->iSarrayKeyExists('quote', $detailsarray['quotes'])) {
                foreach ($detailsarray['rates'] as $dkey => $dvalue) {
                    //NOTE: carrier id can be same in few rates so check id and price!
                    if ($dvalue['servicelevel']->token == $selected['servicelevel']->token and $dvalue['amount'] == $selected['amount']) {
                        $found = true;
                    }
                }
            }
            //if found check currency id
            if (!$found) {
                $service_rate->selected = '';
            }
        }
        if (!$service_rate->save()) {
            return false;
        }

        return $service_rate;
    }

    public function ajaxProcessGetOriginsDirections()
    {
        $api_key = Tools::getValue('api_key');
        
        try {
            require_once dirname(__FILE__).'/api/loader.php';

            Enviaya::setApiKey($api_key);

            $response =  Enviaya_Direction::all(array(
                'api_key' => $api_key,
                'get_destinations' => 'false',
            ));
            
            die(Tools::JsonEncode($response->__toArray(true)));

        }catch (exception $e) {

            $error_msg = $e->getmessage();
            if (class_exists('Logger')) {
                PrestaShopLogger::addLog('[EnviaYa] API GET directions request fails: '.Tools::jsonEncode($error_msg), 3, null, '', 0, true);
            }
            header('HTTP/1.1 500 Internal Server Booboo');
            header('Content-Type: application/json; charset=UTF-8');
            die(Tools::jsonEncode($error_msg));
            return false;

        }
        
    }

    public function ajaxProcessGetBillingAccounts()
    {

        $api_key = Tools::getValue('api_key');
       
        try {
            require_once dirname(__FILE__).'/api/loader.php';

            Enviaya::setApiKey($api_key);

            $response =  Enviaya_Account::all(array(
                'api_key' => $api_key
            ));

            die(Tools::JsonEncode($response->__toArray(true)));

        }catch (exception $e) {
            $error_msg = $e->getmessage();
            if (class_exists('Logger')) {
                PrestaShopLogger::addLog('[EnviaYa] API GET billing  accounts request fails: '.Tools::jsonEncode($error_msg), 3, null, '', 0, true);
            }
            header('HTTP/1.1 500 Internal Server Booboo');
            header('Content-Type: application/json; charset=UTF-8');
            die(Tools::jsonEncode($error_msg));
            return false;
        }
       
        
    }

    public function ajaxProcessGetTracking(){

        $shipment_number = Tools::getValue('shipment_number');
        $order_id = Tools::getValue('order_id');

        try {

            require_once dirname(__FILE__).'/api/loader.php';

            $is_test_mode = (int) Configuration::get('ENVIAYA_TEST_MODE');
            if ($is_test_mode) {
                $api_key =  Configuration::get('ENVIAYA_TEST_API_KEY');
            } else {
                $api_key =  Configuration::get('ENVIAYA_LIVE_API_KEY');
            }

            Enviaya::setApiKey($api_key);

            $order = new Order((int)$order_id);

            $cart = new Cart($order->id_cart);
            $service_rate = EnviayaServiceRate::getFromCart($cart->id);
            $booking = unserialize($service_rate->getBooking());

            $response = Enviaya_Track::create(array(
                'api_key' => $api_key,
                'shipment_number' => $shipment_number,
                'carrier' => $booking->carrier,
            ));

            if (is_object($response))  {


                $response->__set('last_update', date('Y-m-dTH:i:s') );
                $serializedTracking = serialize($response);
              
                $service_rate->extra_data = Tools::jsonEncode($serializedTracking);
                $service_rate->save();

                header('Content-Type: application/json; charset=UTF-8');
                return die(Tools::JsonEncode(array(
                    'tracking' => $response->__toArray(true),
                    'last_update' => date( Context::getContext()->language->date_format_full, strtotime($response->last_update)  )
                )));

           
            }

        }catch (exception $e) {
            $error_msg = $e->getmessage();
            
            header('HTTP/1.1 500 Internal Server Booboo');
            header('Content-Type: application/json; charset=UTF-8');
            die(Tools::jsonEncode($error_msg));
            return false;
        }
    }
    
    public function ajaxProcessGetRates(){

        $order_id = Tools::getValue('order_id');
   
        try {

            require_once dirname(__FILE__).'/api/loader.php';

            $is_test_mode = (int) Configuration::get('ENVIAYA_TEST_MODE');
            if ($is_test_mode) {
                $api_key =  Configuration::get('ENVIAYA_TEST_API_KEY');
            } else {
                $api_key =  Configuration::get('ENVIAYA_LIVE_API_KEY');
            }

            Enviaya::setApiKey($api_key);
           
            $order = new Order((int)$order_id);
            
            if (Validate::isLoadedObject($order) && $order->current_state != Configuration::get('PS_OS_ERROR')) {

                $cart = new Cart($order->id_cart);
                $service_rate = EnviayaServiceRate::getFromCart($cart->id);
                
                if (!$service_rate->id || empty($service_rate->requestsaved)) {
   
                    return false;

                }else{

                    $quotedata = $service_rate->requestsaved;
                    
                    $response = Enviaya_Rate::rating(json_decode($quotedata,true));

                    if ($response) {

                        $serializedObject = serialize($response);

                        $service_rate->details = Tools::jsonEncode($serializedObject);

                        $response_arr = $response->__toArray(true);
                        
                        
                        
                        
                        $price = array();
                        foreach ($response_arr['rates'] as $keyRate => $row)
                        {

                            if( isset($row['additional_configuration']['free_shipping'])  || (string)$keyRate=="store_pickup" ){
                                
                                $price[$keyRate] = 0.00;
                            }else{

                                $price[$keyRate] = $row['total_amount'];
                                
                            }

                        }

                        array_multisort($price, SORT_ASC, $response_arr['rates']);

                        if (!empty($response) and !empty($response_arr['rates']) and array_key_exists(0, $response_arr['rates'])) {

                            $quote = Enviaya_Util::convertToEnviayaObject($response_arr['rates'][0],$api_key);

                            $serializedSelected = serialize($quote);
                            $service_rate->selected = Tools::jsonEncode($serializedSelected);
                            $service_rate->save();
                            
                            header('Content-Type: application/json; charset=UTF-8');
                            return die(Tools::JsonEncode($response_arr));
 
                        

                        }

                    }
                    
                }

            }

        } catch (exception $e) {
            $error_msg = $e->getmessage();
            
            header('HTTP/1.1 500 Internal Server Booboo');
            header('Content-Type: application/json; charset=UTF-8');
            die(Tools::jsonEncode($error_msg));
            return false;
        }
    }

    public function ajaxProcessCreateShipment(){

        $order_id = Tools::getValue('order_id');
        $rate_id = Tools::getValue('rate_id');


        try {

            require_once dirname(__FILE__).'/api/loader.php';

            $is_test_mode = (int) Configuration::get('ENVIAYA_TEST_MODE');
            if ($is_test_mode) {
                $api_key =  Configuration::get('ENVIAYA_TEST_API_KEY');
            } else {
                $api_key =  Configuration::get('ENVIAYA_LIVE_API_KEY');
            }
            
            Enviaya::setApiKey($api_key);

            $createshipment_data = $this->collectdata($order_id,$rate_id);
            
            $order = new Order((int)$order_id);
            if (!Validate::isLoadedObject($order)) {
                return false;
            }

            $request = array_merge(
                array(
                    'api_key' => $api_key,
                    'enviaya_account' => Configuration::get('ENVIAYA_ACCOUNT'),
                    'api_application_id' => 6
                ),
                $createshipment_data
            );

            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                PrestaShopLogger::addLog('[EnviaYa] API create shipment request JUST before call: '.Tools::jsonEncode($request), 1);
            }
            

            $response = Enviaya_Shipment::create($request);
            if (class_exists('Logger') && Configuration::get('ENVIAYA_LOG')) {
                if (is_object($response)) {
                    //if rates returned should be an Object so Tools::jsonEncode will not print anything
                    PrestaShopLogger::addLog('[EnviaYa] API create shipment response: '.print_r($response, true), 1);
                } else {
                    //if error
                    PrestaShopLogger::addLog('[EnviaYa] API create shipment response: '.Tools::jsonEncode($response), 1);
                }
            }
            
            if (is_object($response)) {
                if ($response['id']) {
                    $service_rate = EnviayaServiceRate::getFromCart($order->id_cart); //?order id and cart id are not the same!
                    $serializedOrder = serialize($response);
                    $service_rate->booking = Tools::jsonEncode($serializedOrder);
                    $service_rate->save();
                }
                $result = true;
            } else {
                $result = false;
            }
            return die(Tools::JsonEncode(array('result' => $result)));

           

        } catch (exception $e) {
            $error_msg = $e->getmessage();
            header('HTTP/1.1 500 Internal Server Booboo');
            header('Content-Type: application/json; charset=UTF-8');
            die(Tools::jsonEncode($error_msg));
            return false;
        }


    }

    public function buildParcels($products, $order_currency = null){

        $parcels = array();
       
        if( in_array(Configuration::get('PS_WEIGHT_UNIT'),array('kg','lbs')) && in_array(Configuration::get('PS_DIMENSION_UNIT'),array('cm','in')) ){

            foreach($products as $product){
                
                if($product['is_virtual'] == 0){

                    $qty = isset($product['cart_quantity']) ? $product['cart_quantity'] :  $product['product_quantity'];
                    $weight = ( $product['weight'] > 0 ) ? round($product['weight'], 2) : null;
                    $length = ( $product['depth'] > 0 ) ? round($product['depth'], 2) : null;
                    $width = ( $product['width'] > 0 ) ? round($product['width'], 2) : null;
                    $height = ( $product['height'] > 0 ) ? round($product['height'], 2) : null;
                    
                    if(!is_null($weight)){
                        $parcel = array(
                            'quantity'      => $qty,
                            'weight'        => $weight,
                            'weight_unit'   => Configuration::get('PS_WEIGHT_UNIT'),
                        );
                    }

                    if( isset($parcel) && !is_null($length) && !is_null($width) && !is_null($height) ){
                        
                        $parcel['length'] = $length;
                        $parcel['height'] = $height;
                        $parcel['width'] = $width;
                        $parcel['dimension_unit'] = Configuration::get('PS_DIMENSION_UNIT');

                    }

                    
                    
                    if(!empty($parcel)){
                        
                        $parcel['content'] = isset($product['name']) ? $product['name'] : $product['product_name'];
                        $parcel['value'] = (float) $this->getRounddNumber($product['total_wt'],2);
                        $parcel['value_currency'] = $order_currency->iso_code; 
                        
                        $parcels[] = $parcel;
                    }

                }else{
                    return array(); 
                }
            }

        }
       
        return $parcels;

        
        
    }

    public function buildCommodities($products, $order_currency = null){

        $commodities = array();

        if( in_array(Configuration::get('PS_WEIGHT_UNIT'),array('kg','lbs')) ){

            $country_of_manufacture = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));


            foreach($products as $product){

                if($product['is_virtual'] == 0){

                    $qty = isset($product['cart_quantity']) ? $product['cart_quantity'] :  $product['product_quantity'];
                    $weight = ( $product['weight'] > 0 ) ? round($product['weight'], 2) : null;


                    if(!is_null($weight)){

                        $commodity = array(
                            'quantity' => $qty,
                            'description' => isset($product['name']) ? $product['name'] : $product['product_name'],
                            'unit_price' => (float) $this->getRounddNumber($product['unit_price_tax_incl'],2),
                            'unit_price_currency' =>  $order_currency->iso_code,
                            'total_customs_value' => (float) $this->getRounddNumber($product['total_price_tax_incl'],2), 
                            'total_customs_value_currency' => $order_currency->iso_code,
                            'weight_per_piece' => $weight,
                            'weight_per_piece_unit' => Configuration::get('PS_WEIGHT_UNIT'),
                            'country_of_manufacture' => $country_of_manufacture,
                        );

                        $commodities[] = $commodity;

                    }

                }else{
                    return array(); 
                }
            }

        }

        return $commodities;
        
    }
    //


    public function cleanString($text)
    {
        if (Tools::strlen((string)$text) > 0) {
            $text0 = $this->remove_accents($text);
           
           

            return $text0;
        } else {
            return '';
        }
    }

    public function cleaner($text)
    {
        $text = preg_replace('/&([a-z, ])(acute|uml|circ|grave|ring|cedil|slash|tilde|caron|lig|quot|rsquo);/i', '\\1', $text);
        $text = preg_replace('/[^A-Za-z0-9, \-]/', '', $text);
        $text = preg_replace('/[^\x20-\x7E]*/', '', $text);

        return $text;
    }

    public function iSarrayKeyExists($key, $array)
    {
        if (is_array($array) and array_key_exists($key, $array)) {
            $text = $array[$key];
        } else {
            $text = '';
        }

        return $text;
    }

    public function iSobjctKeyExists($key, $object)
    {
        //if (is_object($object) and property_exists($object, $key)) {
        if (is_object($object) and isset($object->$key)) {
            $text = $object->$key; // array returned if messages
        } else {
            $text = '';
        }

        return $text;
    }

    public static function isEnviayaCarrier($idCarrierReference)
    {
        //check is $idCarrierReference our carrier
        $sql = 'SELECT `id_carrier` FROM `'._DB_PREFIX_.'carrier` WHERE `id_carrier`= '.(int) $idCarrierReference.' AND `external_module_name`="enviayacarrier" AND `active`=1';
        $result = Db::getInstance()->getValue($sql); //Db::getInstance()->execute($sql);
        if ($result) {
            if ((int) $result > 0) {
                return true; //id_carrier returned
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    public static function getEnviayaCarrier()
    {
        //return our carrier id
        //dear validation team. read it carefully. No PSQL needed because sql is pure static text.
        $sql = 'SELECT `id_carrier` FROM `'._DB_PREFIX_.'carrier` WHERE `external_module_name`="enviayacarrier" AND `active`=1';
        //$result = Db::getInstance()->execute($sql); //TRUE
        $result = Db::getInstance()->executeS($sql);

        if ($result) {
            foreach ($result as $key => $row) {
                $carrier_id = $row['id_carrier'];
            }

            return $carrier_id;
        } else {
            return false;
        }
    }

    public function getRounddNumber($value, $round = 2)
    {
        return number_format(Tools::ps_round(($value), (int) $round), (int) $round, '.', '');
    }

    //prepare data for create shipment call
    public function collectdata($order_id, $rate_id)
    {
       
        $order_data = $destination_direction = $origin_direction = $shipment = array();
        $order = new Order((int)$order_id);
        if (Validate::isLoadedObject($order) && $order->current_state != Configuration::get('PS_OS_ERROR')) {

            $cart = new Cart($order->id_cart);
            $customer = new Customer($order->id_customer);
            $carrier = new Carrier($order->id_carrier);
            $products = $this->getOrderProducts($order);
            $currency = new Currency((int) $order->id_currency);
            
            $carrier = new Carrier((int) $order->id_carrier); //$params['cart']->id_carrier
            $idCarrierReference = $carrier->id_reference;

            if (false === $this->isEnviayaCarrier($idCarrierReference)) {

                return false;
            }
            
            $service_rate = EnviayaServiceRate::getFromCart($cart->id);
            $service_rate_id = $service_rate->id;
            
            if (!$service_rate->id || !$service_rate->details) {

                return false;
            } else {

                $ratesreceived = unserialize(Tools::jsonDecode($service_rate->details, true));
                
                if (!empty($ratesreceived['rates'])) {
                    foreach ($ratesreceived['rates'] as $quotekey => $qtvalue) {
                        
                        if($rate_id == $qtvalue->rate_id){
                            $rate_selected = $qtvalue;
                            break;
                        }

                    } 
                }

                $customerAddress = new Address((int) $order->id_address_delivery);
                $customerAddressIso = $iso = Country::getIsoById((int) $customerAddress->id_country);

                // Retrieve addresses information
                $addressInvoice = new Address($order->id_address_invoice, $this->context->language->id);
                if ($order->id_address_invoice == $order->id_address_delivery) {
                    $addressDelivery = $addressInvoice;
                } else {
                    $addressDelivery = new Address($order->id_address_delivery, $this->context->language->id);
                }
                $deliveryState = new State((int) ($addressDelivery->id_state));

                if (!Validate::isLoadedObject($addressDelivery)) {

                    return false;
                }

                $origin_direction = array_filter(array(
                    'full_name' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_FULL_NAME')),
                    'company' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_COMPANY')),
                    'postal_code' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_POSTAL_CODE')),
                    'direction_1' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_1')),
                    'direction_2' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_DIRECTION_2')),
                    'city' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_CITY')),
                    'state_code' => strtoupper($this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_STATE_CODE'))),
                    'country_code' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_COUNTRY_CODE')),
                    'neighborhood' => $this->cleanString(Configuration::get('ENVIAYA_SENDER_ADDRESS_NEIGHBORHOOD')),
                    'phone' => $this->cleaner(Configuration::get('ENVIAYA_SENDER_ADDRESS_PHONE')),
                    'email' => Configuration::get('ENVIAYA_SENDER_ADDRESS_EMAIL'),
                ), function($value) { return !is_null($value) && $value !== ''; });

                if( isset($origin_direction['neighborhood']) && isset($origin_direction['direction_1']) ){
                    
                    if( strpos(strtoupper($origin_direction['direction_1']), strtoupper($origin_direction['neighborhood'])) != false){
                        $origin_direction['direction_1'] = explode(',',$origin_direction['direction_1'])[0];
                    }
                }

                $destination_direction = array_filter(array(
                    'full_name' => $this->cleanString($customer->firstname.' '.$customer->lastname),
                    'company' => $this->cleanString($addressDelivery->company),
                    'postal_code' => $this->cleanString($addressDelivery->postcode),
                    'direction_1' => $this->cleanString($addressDelivery->address1),
                    'direction_2' => $this->cleanString((string)$addressDelivery->address2),
                    'city' => $this->cleanString($addressDelivery->city),
                    'state_code' => isset($deliveryState) ? $deliveryState->iso_code : null,
                    'country_code' => $this->cleanString(Country::getIsoById((int)$addressDelivery->id_country)),
                    'phone' => isset($addressDelivery->phone) ? $this->cleaner($addressDelivery->phone) : $this->cleaner($addressDelivery->phone_mobile),
                    'email' => $customer->email,
                ), function($value) { return !is_null($value) && $value !== ''; });

                
                $products = $this->getOrderProducts($order);
                
                $parcels = $this->buildParcels($products, $currency);
                $commodities = $this->buildCommodities($products, $currency);

                $order_data['shipment_id'] = $rate_selected->shipment_id;
                $order_data['rate_id'] = $rate_selected->rate_id;
                $order_data['carrier'] = $rate_selected->carrier;
                $order_data['carrier_service_code'] = $rate_selected->carrier_service_code;
                $order_data['origin_direction'] = $origin_direction;
                $order_data['destination_direction'] = $destination_direction;
                $order_data['shipment']['shipment_type'] = 'Package';
                $order_data['shipment']['order_total_amount'] = bcdiv($order->total_products_wt,1,2);
                $order_data['shipment']['currency'] = $currency->iso_code;
                $order_data['shipment']['content'] = 'Varios';

                

                if(!empty($parcels)){
                    $order_data['shipment']['parcels'] = $parcels;
                }

         
                $order_data['customs'] = array(
                    'duties_payment' => 'Recipient',
                    'total_declared_value' => bcdiv($order->total_products_wt,1,2),
                    'total_declared_value_currency' =>  $currency->iso_code,
                    'commodities' => $commodities,           
                );

                
                
                
                $order_data['label_format'] = 'Letter';
                $order_data['locale'] = Language::getLocaleById($order->id_lang);
                $order_data['currency'] = $currency->iso_code;
                $order_data['shop_order_id'] = $order->id;
                $order_data['shop_domain'] = Configuration::get('PS_SHOP_DOMAIN'); 
                
              
                return $order_data;
                
            }

        }

    }

    /**
     * @param Order $order
     *
     * @return array
     */
    protected function getOrderProducts($order)
    {
        $products = $order->getProducts();
       
        foreach ($products as &$product) {
           
            
            if ($product['image'] != null) {
                $name = 'product_mini_' . (int) $product['product_id'] . (isset($product['product_attribute_id']) ? '_' . (int) $product['product_attribute_id'] : '') . '.jpg';
                // generate image cache, only for back office
                $product['image_tag'] = ImageManager::thumbnail(_PS_IMG_DIR_ . 'p/' . $product['image']->getExistingImgPath() . '.jpg', $name, 45, 'jpg');
                if (file_exists(_PS_TMP_IMG_DIR_ . $name)) {
                    $product['image_size'] = getimagesize(_PS_TMP_IMG_DIR_ . $name);
                } else {
                    $product['image_size'] = false;
                }
            }
        }

        ksort($products);

        return $products;
    }

    public function calcDeliveryTime($item)
    {
        if ($item < 24) {
            $result = round($item, 1);

            if ($result == 1) {
                $result = $result . ' ' . $this->l('hour');
            } else {
                $result = $result . ' ' . $this->l('hours');
            }
        } else {
            $result = round($item / 24, 1);

            if ($result == 1) {
                $result = $result . ' ' . $this->l('day');
            } else {
                $result = $result . ' ' .$this->l('days');
            }
        }

        return $result;
    }

    public function getStates($id_lang)
    {


        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
		SELECT value,text FROM (
            SELECT CONCAT("Z",id_zone) value , name "2", NULL "3" , NULL "4" , name text FROM '._DB_PREFIX_.'zone 
            UNION 
            SELECT CONCAT("C",'._DB_PREFIX_.'country.id_country) value, '._DB_PREFIX_.'zone.name "2" , '._DB_PREFIX_.'country_lang.name "3" , NULL  "4",  CONCAT( '._DB_PREFIX_.'zone.name, " / ",'._DB_PREFIX_.'country_lang.name) text FROM '._DB_PREFIX_.'country
            JOIN '._DB_PREFIX_.'country_lang  ON ( '._DB_PREFIX_.'country_lang.id_country = '._DB_PREFIX_.'country.id_country )
            JOIN '._DB_PREFIX_.'zone  ON ( '._DB_PREFIX_.'country.id_zone= '._DB_PREFIX_.'zone.id_zone )
            WHERE id_lang = '.$id_lang.'
            UNION 
            SELECT CONCAT("S",id_state) value, '._DB_PREFIX_.'zone.name "2" , '._DB_PREFIX_.'country_lang.name "3", '._DB_PREFIX_.'state.name  "4",  CONCAT( '._DB_PREFIX_.'zone.name, " / ",'._DB_PREFIX_.'country_lang.name, " / ",'._DB_PREFIX_.'state.name) text FROM '._DB_PREFIX_.'state
            JOIN '._DB_PREFIX_.'zone  ON ( '._DB_PREFIX_.'state.id_zone= '._DB_PREFIX_.'zone.id_zone )
            JOIN '._DB_PREFIX_.'country_lang  ON ( '._DB_PREFIX_.'state.id_country = '._DB_PREFIX_.'country_lang.id_country )
            WHERE id_lang = '.$id_lang.'
            ORDER BY 2,3,4
            ) regions');
		
    }

    function remove_accents($string) {
        if ( !preg_match('/[\x80-\xff]/', $string) )
            return $string;
    
        $chars = array(
        // Decompositions for Latin-1 Supplement
        chr(195).chr(128) => 'A', chr(195).chr(129) => 'A',
        chr(195).chr(130) => 'A', chr(195).chr(131) => 'A',
        chr(195).chr(132) => 'A', chr(195).chr(133) => 'A',
        chr(195).chr(135) => 'C', chr(195).chr(136) => 'E',
        chr(195).chr(137) => 'E', chr(195).chr(138) => 'E',
        chr(195).chr(139) => 'E', chr(195).chr(140) => 'I',
        chr(195).chr(141) => 'I', chr(195).chr(142) => 'I',
        chr(195).chr(143) => 'I', chr(195).chr(145) => 'N',
        chr(195).chr(146) => 'O', chr(195).chr(147) => 'O',
        chr(195).chr(148) => 'O', chr(195).chr(149) => 'O',
        chr(195).chr(150) => 'O', chr(195).chr(153) => 'U',
        chr(195).chr(154) => 'U', chr(195).chr(155) => 'U',
        chr(195).chr(156) => 'U', chr(195).chr(157) => 'Y',
        chr(195).chr(159) => 's', chr(195).chr(160) => 'a',
        chr(195).chr(161) => 'a', chr(195).chr(162) => 'a',
        chr(195).chr(163) => 'a', chr(195).chr(164) => 'a',
        chr(195).chr(165) => 'a', chr(195).chr(167) => 'c',
        chr(195).chr(168) => 'e', chr(195).chr(169) => 'e',
        chr(195).chr(170) => 'e', chr(195).chr(171) => 'e',
        chr(195).chr(172) => 'i', chr(195).chr(173) => 'i',
        chr(195).chr(174) => 'i', chr(195).chr(175) => 'i',
        chr(195).chr(177) => 'n', chr(195).chr(178) => 'o',
        chr(195).chr(179) => 'o', chr(195).chr(180) => 'o',
        chr(195).chr(181) => 'o', chr(195).chr(182) => 'o',
        chr(195).chr(182) => 'o', chr(195).chr(185) => 'u',
        chr(195).chr(186) => 'u', chr(195).chr(187) => 'u',
        chr(195).chr(188) => 'u', chr(195).chr(189) => 'y',
        chr(195).chr(191) => 'y',
        // Decompositions for Latin Extended-A
        chr(196).chr(128) => 'A', chr(196).chr(129) => 'a',
        chr(196).chr(130) => 'A', chr(196).chr(131) => 'a',
        chr(196).chr(132) => 'A', chr(196).chr(133) => 'a',
        chr(196).chr(134) => 'C', chr(196).chr(135) => 'c',
        chr(196).chr(136) => 'C', chr(196).chr(137) => 'c',
        chr(196).chr(138) => 'C', chr(196).chr(139) => 'c',
        chr(196).chr(140) => 'C', chr(196).chr(141) => 'c',
        chr(196).chr(142) => 'D', chr(196).chr(143) => 'd',
        chr(196).chr(144) => 'D', chr(196).chr(145) => 'd',
        chr(196).chr(146) => 'E', chr(196).chr(147) => 'e',
        chr(196).chr(148) => 'E', chr(196).chr(149) => 'e',
        chr(196).chr(150) => 'E', chr(196).chr(151) => 'e',
        chr(196).chr(152) => 'E', chr(196).chr(153) => 'e',
        chr(196).chr(154) => 'E', chr(196).chr(155) => 'e',
        chr(196).chr(156) => 'G', chr(196).chr(157) => 'g',
        chr(196).chr(158) => 'G', chr(196).chr(159) => 'g',
        chr(196).chr(160) => 'G', chr(196).chr(161) => 'g',
        chr(196).chr(162) => 'G', chr(196).chr(163) => 'g',
        chr(196).chr(164) => 'H', chr(196).chr(165) => 'h',
        chr(196).chr(166) => 'H', chr(196).chr(167) => 'h',
        chr(196).chr(168) => 'I', chr(196).chr(169) => 'i',
        chr(196).chr(170) => 'I', chr(196).chr(171) => 'i',
        chr(196).chr(172) => 'I', chr(196).chr(173) => 'i',
        chr(196).chr(174) => 'I', chr(196).chr(175) => 'i',
        chr(196).chr(176) => 'I', chr(196).chr(177) => 'i',
        chr(196).chr(178) => 'IJ',chr(196).chr(179) => 'ij',
        chr(196).chr(180) => 'J', chr(196).chr(181) => 'j',
        chr(196).chr(182) => 'K', chr(196).chr(183) => 'k',
        chr(196).chr(184) => 'k', chr(196).chr(185) => 'L',
        chr(196).chr(186) => 'l', chr(196).chr(187) => 'L',
        chr(196).chr(188) => 'l', chr(196).chr(189) => 'L',
        chr(196).chr(190) => 'l', chr(196).chr(191) => 'L',
        chr(197).chr(128) => 'l', chr(197).chr(129) => 'L',
        chr(197).chr(130) => 'l', chr(197).chr(131) => 'N',
        chr(197).chr(132) => 'n', chr(197).chr(133) => 'N',
        chr(197).chr(134) => 'n', chr(197).chr(135) => 'N',
        chr(197).chr(136) => 'n', chr(197).chr(137) => 'N',
        chr(197).chr(138) => 'n', chr(197).chr(139) => 'N',
        chr(197).chr(140) => 'O', chr(197).chr(141) => 'o',
        chr(197).chr(142) => 'O', chr(197).chr(143) => 'o',
        chr(197).chr(144) => 'O', chr(197).chr(145) => 'o',
        chr(197).chr(146) => 'OE',chr(197).chr(147) => 'oe',
        chr(197).chr(148) => 'R',chr(197).chr(149) => 'r',
        chr(197).chr(150) => 'R',chr(197).chr(151) => 'r',
        chr(197).chr(152) => 'R',chr(197).chr(153) => 'r',
        chr(197).chr(154) => 'S',chr(197).chr(155) => 's',
        chr(197).chr(156) => 'S',chr(197).chr(157) => 's',
        chr(197).chr(158) => 'S',chr(197).chr(159) => 's',
        chr(197).chr(160) => 'S', chr(197).chr(161) => 's',
        chr(197).chr(162) => 'T', chr(197).chr(163) => 't',
        chr(197).chr(164) => 'T', chr(197).chr(165) => 't',
        chr(197).chr(166) => 'T', chr(197).chr(167) => 't',
        chr(197).chr(168) => 'U', chr(197).chr(169) => 'u',
        chr(197).chr(170) => 'U', chr(197).chr(171) => 'u',
        chr(197).chr(172) => 'U', chr(197).chr(173) => 'u',
        chr(197).chr(174) => 'U', chr(197).chr(175) => 'u',
        chr(197).chr(176) => 'U', chr(197).chr(177) => 'u',
        chr(197).chr(178) => 'U', chr(197).chr(179) => 'u',
        chr(197).chr(180) => 'W', chr(197).chr(181) => 'w',
        chr(197).chr(182) => 'Y', chr(197).chr(183) => 'y',
        chr(197).chr(184) => 'Y', chr(197).chr(185) => 'Z',
        chr(197).chr(186) => 'z', chr(197).chr(187) => 'Z',
        chr(197).chr(188) => 'z', chr(197).chr(189) => 'Z',
        chr(197).chr(190) => 'z', chr(197).chr(191) => 's'
        );
    
        $string = strtr($string, $chars);
    
        return $string;
    }
}


